#Copyright ReportLab Europe Ltd. 2000-2017
#see license.txt for license details
"""Tests ability to cycle through multiple page templates
"""
__version__='3.3.0'
from reportlab.lib.testutils import setOutDir,makeSuiteForClasses, outputfile, printLocation
setOutDir(__name__)
import sys, os, time, random
from reportlab.rl_config import invariant as rl_invariant
from operator import truth
import unittest
from reportlab.platypus.flowables import Flowable
from reportlab.lib import colors
from reportlab.lib.pagesizes import A4
from reportlab.lib.units import cm
from reportlab.lib.enums import TA_LEFT, TA_RIGHT, TA_CENTER, TA_JUSTIFY
from reportlab.lib.styles import getSampleStyleSheet, ParagraphStyle
from reportlab.platypus.paragraph import Paragraph
from reportlab.platypus.frames import Frame
from reportlab.lib.randomtext import randomText, PYTHON
from reportlab.platypus.doctemplate import PageTemplate, BaseDocTemplate, NextPageTemplate
from reportlab.platypus.paragraph import *


def myMainPageFrame(canvas, doc):
    "The page frame used for all PDF documents."

    canvas.saveState()
    canvas.setFont('Times-Roman', 12)
    pageNumber = canvas.getPageNumber()
    canvas.drawString(10*cm, cm, str(pageNumber))
    canvas.restoreState()

class LeftPageTemplate(PageTemplate):
    def __init__(self):
        #allow a bigger margin on the right for the staples
        frame = Frame(1.5*cm, 2.5*cm, 16*cm, 25*cm, id='F1')

        PageTemplate.__init__(self,
                              id='left',
                              frames=[frame],
                              pagesize=A4)
    def beforeDrawPage(self, canv, doc):
        "Decorate the page with an asymetric design"
        canv.setFillColor(colors.cyan)
                          
        canv.rect(0.5*cm, 2.5*cm, 1*cm, 25*cm, stroke=1, fill=1)
        canv.circle(19*cm, 10*cm, 0.5*cm, stroke=1, fill=1)
        canv.circle(19*cm, 20*cm, 0.5*cm, stroke=1, fill=1)
        canv.setFillColor(colors.black)
        

class RightPageTemplate(PageTemplate):
    def __init__(self):
        #allow a bigger margin on the right for the staples
        frame = Frame(3.5*cm, 2.5*cm, 16*cm, 25*cm, id='F1')

        PageTemplate.__init__(self,
                              id='right',
                              frames=[frame],
                              pagesize=A4)
    def beforeDrawPage(self, canv, doc):
        "Decorate the page with an asymetric design"
        canv.setFillColor(colors.cyan)
        canv.rect(19.5*cm, 2.5*cm, 1*cm, 25*cm, stroke=1, fill=1)
        canv.circle(2*cm, 10*cm, 0.5*cm, stroke=1, fill=1)
        canv.circle(2*cm, 20*cm, 0.5*cm, stroke=1, fill=1)
        canv.setFillColor(colors.black)


class MyDocTemplate(BaseDocTemplate):
    _invalidInitArgs = ('pageTemplates',)

    def __init__(self, filename, **kw):
        BaseDocTemplate.__init__(self, filename, **kw)
        self.addPageTemplates(
            [
             PageTemplate(id='plain',
                          frames=[Frame(2.5*cm, 2.5*cm, 16*cm, 25*cm, id='F1')]
                          ),
             LeftPageTemplate(),
             RightPageTemplate()
            ]
            )


class LeftRightTestCase(unittest.TestCase):
    "Test multi-page splitting of paragraphs (eyeball-test)."
    def testIt(self):
        "LeftRightTestCase.testit"

        if rl_invariant: random.seed(464568862)
        # Build story.
        story = []

        styleSheet = getSampleStyleSheet()
        h1 = styleSheet['Heading1']
        h1.pageBreakBefore = 1
        h1.keepWithNext = 1

        h2 = styleSheet['Heading2']
        h2.frameBreakBefore = 1
        h2.keepWithNext = 1

        h3 = styleSheet['Heading3']
        h3.backColor = colors.cyan
        h3.keepWithNext = 1

        bt = styleSheet['BodyText']

        story.append(Paragraph("""
            This tests ability to alternate left and right templates.  We start on
            a plain one. The next page should display a left-side template,
            with a big inner margin and staple-like holes on the right.""",style=bt))

        story.append(NextPageTemplate(['left','right']))

        story.append(Paragraph("""
            One can specify a list of templates instead of a single one in
            order to sequence through them.""",style=bt))
        def doSome():
            for i in range(10):
                story.append(Paragraph('Heading 1 always starts a new page (%d)' % len(story), h1))
                for j in range(3):
                    story.append(Paragraph('Heading1 paragraphs should always'
                                    'have a page break before.  Heading 2 on the other hand'
                                    'should always have a FRAME break before (%d)' % len(story), bt))
                    story.append(Paragraph('Heading 2 always starts a new frame (%d)' % len(story), h2))
                    story.append(Paragraph('Heading1 paragraphs should always'
                                    'have a page break before.  Heading 2 on the other hand'
                                    'should always have a FRAME break before (%d)' % len(story), bt))
                    for j in range(3):
                        story.append(Paragraph(randomText(theme=PYTHON, sentences=2)+' (%d)' % len(story), bt))
                        story.append(Paragraph('I should never be at the bottom of a frame (%d)' % len(story), h3))
                        story.append(Paragraph(randomText(theme=PYTHON, sentences=1)+' (%d)' % len(story), bt))

        doSome()
        story.append(NextPageTemplate('plain'))
        story.append(Paragraph('Back to plain old page template',h1))
        story.append(Paragraph('Back to plain old formatting', bt))
        story.append(Paragraph("""use a template name of * to indicate where the iteration should restart""",style=bt))
        story.append(NextPageTemplate(['left','*','right']))
        doSome()

        #doc = MyDocTemplate(outputfile('test_platypus_leftright.pdf'))
        doc = MyDocTemplate(outputfile('test_platypus_leftright.pdf'))
        doc.multiBuild(story)

def makeSuite():
    return makeSuiteForClasses(LeftRightTestCase)


#noruntests
if __name__ == "__main__": #NORUNTESTS
    unittest.TextTestRunner().run(makeSuite())
    printLocation()



#!/bin/env python
#Copyright ReportLab Europe Ltd. 2000-2017
#see license.txt for license details
__version__='3.3.0'
__doc__="""Test reportlab.lib.util module"""
from reportlab.lib.testutils import setOutDir,makeSuiteForClasses, outputfile, printLocation
setOutDir(__name__)
import unittest
from reportlab.lib.utils import FmtSelfDict, encode_label, decode_label

class FmtTestCase(unittest.TestCase):

    def testFmt(self):
        class MixedIn(FmtSelfDict):
            def __init__(self):
                self.a = 'AA'
                self._b = '_BB'
                self.d = '(overridden)'
        obj = MixedIn()
        self.assertEqual('blah', obj._fmt('blah'))
        self.assertEqual('blah %', obj._fmt('blah %%'))
        self.assertRaises(ValueError, obj._fmt, 'blah %')
        self.assertEqual(
            'moon AA june_BB spoon %(a)sCC ni',
            obj._fmt('moon %(a)s june%(_b)s spoon %%(a)s%(c)s %(d)s', c='CC', C='boon', d='ni'))
        self.assertRaises(AttributeError, obj._fmt, '%(c)s')  # XXX bit weird, can this be changed?

    def testLabelCoding(self):
        a = (1,2,[3,4],"a"), [1,2,"4",b'4']
        assert a == decode_label(encode_label(a))

def makeSuite():
    return makeSuiteForClasses(FmtTestCase)


#noruntests
if __name__ == "__main__":
    unittest.TextTestRunner().run(makeSuite())
    printLocation()



#Autogenerated by ReportLab guiedit do not edit
from reportlab.graphics.shapes import _DrawingEditorMixin, Drawing, Group, Rect, PolyLine, Polygon, Line, String
from reportlab.lib.colors import Color, CMYKColor, PCMYKColor

class ExplodedDrawing_Drawing(_DrawingEditorMixin,Drawing):
	def __init__(self,width=300,height=150,*args,**kw):
		Drawing.__init__(self,width,height,*args,**kw)
		self.transform = (1,0,0,1,0,0)
		self.add(Rect(0,0,300,150,rx=0,ry=0,fillColor=None,fillOpacity=None,strokeColor=Color(1,0,0,1),strokeWidth=1,strokeLineCap=0,strokeLineJoin=0,strokeMiterLimit=0,strokeDashArray=None,strokeOpacity=None))
		self.add(PolyLine(points=[42.5,52.5,87.5,66.66667,132.5,80.83333,177.5,95],strokeColor=Color(1,0,0,1),strokeWidth=1,strokeLineCap=0,strokeLineJoin=1,strokeMiterLimit=0,strokeDashArray=None,strokeOpacity=None))
		self.add(PolyLine(points=[42.5,10,87.5,24.16667,132.5,24.16667,177.5,38.33333],strokeColor=Color(0,.501961,0,1),strokeWidth=1,strokeLineCap=0,strokeLineJoin=1,strokeMiterLimit=0,strokeDashArray=None,strokeOpacity=None))
		self.add(Polygon(points=[39.5,52.5,42.5,55.5,45.5,52.5,42.5,49.5],fillColor=Color(1,0,0,1),fillOpacity=None,strokeColor=Color(0,0,0,1),strokeWidth=.1,strokeLineCap=0,strokeLineJoin=0,strokeMiterLimit=0,strokeDashArray=None,strokeOpacity=None))
		self.add(Polygon(points=[84.5,66.66667,87.5,69.66667,90.5,66.66667,87.5,63.66667],fillColor=Color(1,0,0,1),fillOpacity=None,strokeColor=Color(0,0,0,1),strokeWidth=.1,strokeLineCap=0,strokeLineJoin=0,strokeMiterLimit=0,strokeDashArray=None,strokeOpacity=None))
		self.add(Polygon(points=[129.5,80.83333,132.5,83.83333,135.5,80.83333,132.5,77.83333],fillColor=Color(1,0,0,1),fillOpacity=None,strokeColor=Color(0,0,0,1),strokeWidth=.1,strokeLineCap=0,strokeLineJoin=0,strokeMiterLimit=0,strokeDashArray=None,strokeOpacity=None))
		self.add(Polygon(points=[174.5,95,177.5,98,180.5,95,177.5,92],fillColor=Color(1,0,0,1),fillOpacity=None,strokeColor=Color(0,0,0,1),strokeWidth=.1,strokeLineCap=0,strokeLineJoin=0,strokeMiterLimit=0,strokeDashArray=None,strokeOpacity=None))
		self.add(Rect(40,7.5,5,5,rx=0,ry=0,fillColor=Color(0,.501961,0,1),fillOpacity=None,strokeColor=Color(0,0,0,1),strokeWidth=.1,strokeLineCap=0,strokeLineJoin=0,strokeMiterLimit=0,strokeDashArray=None,strokeOpacity=None))
		self.add(Rect(85,21.66667,5,5,rx=0,ry=0,fillColor=Color(0,.501961,0,1),fillOpacity=None,strokeColor=Color(0,0,0,1),strokeWidth=.1,strokeLineCap=0,strokeLineJoin=0,strokeMiterLimit=0,strokeDashArray=None,strokeOpacity=None))
		self.add(Rect(130,21.66667,5,5,rx=0,ry=0,fillColor=Color(0,.501961,0,1),fillOpacity=None,strokeColor=Color(0,0,0,1),strokeWidth=.1,strokeLineCap=0,strokeLineJoin=0,strokeMiterLimit=0,strokeDashArray=None,strokeOpacity=None))
		self.add(Rect(175,35.83333,5,5,rx=0,ry=0,fillColor=Color(0,.501961,0,1),fillOpacity=None,strokeColor=Color(0,0,0,1),strokeWidth=.1,strokeLineCap=0,strokeLineJoin=0,strokeMiterLimit=0,strokeDashArray=None,strokeOpacity=None))
		self.add(Line(20,10,200,10,strokeColor=Color(0,0,0,1),strokeWidth=1,strokeLineCap=0,strokeLineJoin=0,strokeMiterLimit=0,strokeDashArray=None,strokeOpacity=None))
		self.add(Line(20,10,20,5,strokeColor=Color(0,0,0,1),strokeWidth=1,strokeLineCap=0,strokeLineJoin=0,strokeMiterLimit=10,strokeDashArray=None,strokeOpacity=None))
		self.add(Line(65,10,65,5,strokeColor=Color(0,0,0,1),strokeWidth=1,strokeLineCap=0,strokeLineJoin=0,strokeMiterLimit=10,strokeDashArray=None,strokeOpacity=None))
		self.add(Line(110,10,110,5,strokeColor=Color(0,0,0,1),strokeWidth=1,strokeLineCap=0,strokeLineJoin=0,strokeMiterLimit=10,strokeDashArray=None,strokeOpacity=None))
		self.add(Line(155,10,155,5,strokeColor=Color(0,0,0,1),strokeWidth=1,strokeLineCap=0,strokeLineJoin=0,strokeMiterLimit=10,strokeDashArray=None,strokeOpacity=None))
		self.add(Line(200,10,200,5,strokeColor=Color(0,0,0,1),strokeWidth=1,strokeLineCap=0,strokeLineJoin=0,strokeMiterLimit=10,strokeDashArray=None,strokeOpacity=None))
		self.add(Line(20,10,20,95,strokeColor=Color(0,0,0,1),strokeWidth=1,strokeLineCap=0,strokeLineJoin=0,strokeMiterLimit=0,strokeDashArray=None,strokeOpacity=None))
		self.add(Line(20,10,15,10,strokeColor=Color(0,0,0,1),strokeWidth=1,strokeLineCap=0,strokeLineJoin=0,strokeMiterLimit=10,strokeDashArray=None,strokeOpacity=None))
		self.add(Line(20,24.16667,15,24.16667,strokeColor=Color(0,0,0,1),strokeWidth=1,strokeLineCap=0,strokeLineJoin=0,strokeMiterLimit=10,strokeDashArray=None,strokeOpacity=None))
		self.add(Line(20,38.33333,15,38.33333,strokeColor=Color(0,0,0,1),strokeWidth=1,strokeLineCap=0,strokeLineJoin=0,strokeMiterLimit=10,strokeDashArray=None,strokeOpacity=None))
		self.add(Line(20,52.5,15,52.5,strokeColor=Color(0,0,0,1),strokeWidth=1,strokeLineCap=0,strokeLineJoin=0,strokeMiterLimit=10,strokeDashArray=None,strokeOpacity=None))
		self.add(Line(20,66.66667,15,66.66667,strokeColor=Color(0,0,0,1),strokeWidth=1,strokeLineCap=0,strokeLineJoin=0,strokeMiterLimit=10,strokeDashArray=None,strokeOpacity=None))
		self.add(Line(20,80.83333,15,80.83333,strokeColor=Color(0,0,0,1),strokeWidth=1,strokeLineCap=0,strokeLineJoin=0,strokeMiterLimit=10,strokeDashArray=None,strokeOpacity=None))
		self.add(Line(20,95,15,95,strokeColor=Color(0,0,0,1),strokeWidth=1,strokeLineCap=0,strokeLineJoin=0,strokeMiterLimit=10,strokeDashArray=None,strokeOpacity=None))
		v0=self._nn(Group())
		v0.transform = (1,0,0,1,15,10)
		v0.add(String(-10,-4,'70',textAnchor='start',fontName='Times-Roman',fontSize=10,fillColor=Color(0,0,0,1)))
		v0=self._nn(Group())
		v0.transform = (1,0,0,1,15,24.16667)
		v0.add(String(-10,-4,'80',textAnchor='start',fontName='Times-Roman',fontSize=10,fillColor=Color(0,0,0,1)))
		v0=self._nn(Group())
		v0.transform = (1,0,0,1,15,38.33333)
		v0.add(String(-10,-4,'90',textAnchor='start',fontName='Times-Roman',fontSize=10,fillColor=Color(0,0,0,1)))
		v0=self._nn(Group())
		v0.transform = (1,0,0,1,15,52.5)
		v0.add(String(-15,-4,'100',textAnchor='start',fontName='Times-Roman',fontSize=10,fillColor=Color(0,0,0,1)))
		v0=self._nn(Group())
		v0.transform = (1,0,0,1,15,66.66667)
		v0.add(String(-15,-4,'110',textAnchor='start',fontName='Times-Roman',fontSize=10,fillColor=Color(0,0,0,1)))
		v0=self._nn(Group())
		v0.transform = (1,0,0,1,15,80.83333)
		v0.add(String(-15,-4,'120',textAnchor='start',fontName='Times-Roman',fontSize=10,fillColor=Color(0,0,0,1)))
		v0=self._nn(Group())
		v0.transform = (1,0,0,1,15,95)
		v0.add(String(-15,-4,'130',textAnchor='start',fontName='Times-Roman',fontSize=10,fillColor=Color(0,0,0,1)))
		self.add(Rect(182.78,42,20,5,rx=0,ry=0,fillColor=Color(1,0,0,1),fillOpacity=None,strokeColor=Color(0,0,0,1),strokeWidth=1,strokeLineCap=0,strokeLineJoin=0,strokeMiterLimit=0,strokeDashArray=None,strokeOpacity=None))
		self.add(String(145,40.91,'röt',textAnchor='start',fontName='Helvetica',fontSize=10,fillColor=Color(0,0,0,1)))
		self.add(Rect(182.78,28,20,5,rx=0,ry=0,fillColor=Color(0,0,1,1),fillOpacity=None,strokeColor=Color(0,0,0,1),strokeWidth=1,strokeLineCap=0,strokeLineJoin=0,strokeMiterLimit=0,strokeDashArray=None,strokeOpacity=None))
		self.add(String(145,26.91,'blue',textAnchor='start',fontName='Helvetica',fontSize=10,fillColor=Color(0,0,0,1)))
		self.add(Rect(182.78,14,20,5,rx=0,ry=0,fillColor=Color(0,.501961,0,1),fillOpacity=None,strokeColor=Color(0,0,0,1),strokeWidth=1,strokeLineCap=0,strokeLineJoin=0,strokeMiterLimit=0,strokeDashArray=None,strokeOpacity=None))
		self.add(String(145,12.91,'green',textAnchor='start',fontName='Helvetica',fontSize=10,fillColor=Color(0,0,0,1)))
		self.add(Rect(257.78,42,20,5,rx=0,ry=0,fillColor=Color(1,.752941,.796078,1),fillOpacity=None,strokeColor=Color(0,0,0,1),strokeWidth=1,strokeLineCap=0,strokeLineJoin=0,strokeMiterLimit=0,strokeDashArray=None,strokeOpacity=None))
		self.add(String(220,40.91,'pink',textAnchor='start',fontName='Helvetica',fontSize=10,fillColor=Color(0,0,0,1)))
		self.add(Rect(257.78,28,20,5,rx=0,ry=0,fillColor=Color(1,1,0,1),fillOpacity=None,strokeColor=Color(0,0,0,1),strokeWidth=1,strokeLineCap=0,strokeLineJoin=0,strokeMiterLimit=0,strokeDashArray=None,strokeOpacity=None))
		self.add(String(220,26.91,'yellow',textAnchor='start',fontName='Helvetica',fontSize=10,fillColor=Color(0,0,0,1)))
		self.add(Polygon(points=[294.1667,2.5,294.1667,4.166667,292.5,4.166667,292.5,5.833333,294.1667,5.833333,294.1667,7.5,295.8333,7.5,295.8333,5.833333,297.5,5.833333,297.5,4.166667,295.8333,4.166667,295.8333,2.5],fillColor=Color(1,0,0,1),fillOpacity=None,strokeColor=Color(1,1,0,1),strokeWidth=.1,strokeLineCap=0,strokeLineJoin=0,strokeMiterLimit=0,strokeDashArray=None,strokeOpacity=None))


if __name__=="__main__": #NORUNTESTS
	ExplodedDrawing_Drawing().save(formats=['pdf'],outDir='.',fnRoot=None)



#Autogenerated by ReportLab guiedit do not edit
from reportlab.graphics.shapes import _DrawingEditorMixin, Drawing, Group, Line
from reportlab.lib.colors import Color, CMYKColor, PCMYKColor

class ExplodedDrawing_Drawing(_DrawingEditorMixin,Drawing):
	def __init__(self,width=400,height=200,*args,**kw):
		Drawing.__init__(self,width,height,*args,**kw)
		self.transform = (1,0,0,1,0,0)
		self.add(Line(50,50,300,100,strokeColor=Color(0,0,1,1),strokeWidth=14.17323,strokeLineCap=0,strokeLineJoin=0,strokeMiterLimit=0,strokeDashArray=None,strokeOpacity=None))
		self.add(Line(50,100,300,50,strokeColor=Color(0,0,1,1),strokeWidth=14.17323,strokeLineCap=0,strokeLineJoin=0,strokeMiterLimit=0,strokeDashArray=[5,10,15],strokeOpacity=None))


if __name__=="__main__": #NORUNTESTS
	ExplodedDrawing_Drawing().save(formats=['pdf'],outDir='.',fnRoot=None)



#!/bin/env python
#Copyright ReportLab Europe Ltd. 2000-2017
#see license.txt for license details
#history https://hg.reportlab.com/hg-public/reportlab/log/tip/docs/userguide/genuserguide.py
__version__='3.3.0'
__doc__ = """
This module contains the script for building the user guide.
"""

def run(pagesize=None, verbose=0, outDir=None):
    import sys,os
    from reportlab.lib.utils import open_and_read, asUnicode
    cwd = os.getcwd()
    docsDir=os.path.dirname(os.path.dirname(sys.argv[0]) or cwd)
    topDir=os.path.dirname(docsDir)
    if not outDir: outDir=docsDir
    G = {}
    sys.path.insert(0,topDir)
    from reportlab.pdfbase.pdfmetrics import registerFontFamily
    from reportlab.pdfbase import pdfmetrics
    from reportlab.pdfbase.ttfonts import TTFont
    pdfmetrics.registerFont(TTFont('Vera', 'Vera.ttf'))
    pdfmetrics.registerFont(TTFont('VeraBd', 'VeraBd.ttf'))
    pdfmetrics.registerFont(TTFont('VeraIt', 'VeraIt.ttf'))
    pdfmetrics.registerFont(TTFont('VeraBI', 'VeraBI.ttf'))
    registerFontFamily('Vera',normal='Vera',bold='VeraBd',italic='VeraIt',boldItalic='VeraBI')
    from tools.docco.rl_doc_utils import setStory, getStory, RLDocTemplate, defaultPageSize, H1, H2, H3, H4
    from tools.docco import rl_doc_utils
    exec('from tools.docco.rl_doc_utils import *', G, G)
    destfn = os.path.join(outDir,'reportlab-userguide.pdf')
    doc = RLDocTemplate(destfn,pagesize = pagesize or defaultPageSize)


    #this builds the story
    setStory()

    for f in (
        'ch1_intro',
        'ch2_graphics',
        'ch2a_fonts',
        'ch3_pdffeatures',
        'ch4_platypus_concepts',
        'ch5_paragraphs',
        'ch6_tables',
        'ch7_custom',
        'graph_intro',
        'graph_concepts',
        'graph_charts',
        'graph_shapes',
        'graph_widgets',
        'app_demos',
        ):
        #python source is supposed to be utf8 these days
        exec(asUnicode(open_and_read(f+'.py')), G, G)
    del G

    story = getStory()
    if verbose: print('Built story contains %d flowables...' % len(story))
    doc.multiBuild(story)
    if verbose: print('Saved "%s"' % destfn)

def makeSuite():
    "standard test harness support - run self as separate process"
    from tests.utils import ScriptThatMakesFileTest
    return ScriptThatMakesFileTest('../docs/userguide', 'genuserguide.py', 'reportlab-userguide.pdf')

def main():
    import sys
    outDir = [x for x in sys.argv if x[:9]=='--outdir=']
    if outDir:
        outDir = outDir[0]
        sys.argv.remove(outDir)
        outDir = outDir[9:]
    else:
        outDir = None
    verbose = '-s' not in sys.argv
    if not verbose: sys.argv.remove('-s')
    timing = '-timing' in sys.argv
    if timing: sys.argv.remove('-timing')
    prof = '-prof' in sys.argv
    if prof: sys.argv.remove('-prof')

    if len(sys.argv) > 1:
        try:
            pagesize = (w,h) = eval(sys.argv[1])
        except:
            print('Expected page size in argument 1', sys.argv[1])
            raise
        if verbose:
            print('set page size to',sys.argv[1])
    else:
        pagesize = None
    if timing:
        from time import time
        t0 = time()
        run(pagesize, verbose,outDir)
        if verbose:
            print('Generation of userguide took %.2f seconds' % (time()-t0))
    elif prof:
        import profile
        profile.run('run(pagesize,verbose,outDir)','genuserguide.stats')
    else:
        run(pagesize, verbose,outDir)
if __name__=="__main__":
    main()



#Autogenerated by ReportLab guiedit do not edit
from reportlab.graphics.shapes import _DrawingEditorMixin, Drawing, Group, Rect, Line, String
from reportlab.lib.colors import Color, CMYKColor, PCMYKColor

class ExplodedDrawing_Drawing(_DrawingEditorMixin,Drawing):
	def __init__(self,width=400,height=200,*args,**kw):
		Drawing.__init__(self,width,height,*args,**kw)
		self.transform = (1,0,0,1,0,0)
		self.add(Rect(50,50,300,125,rx=0,ry=0,fillColor=None,fillOpacity=None,strokeColor=Color(0,0,0,1),strokeWidth=1,strokeLineCap=0,strokeLineJoin=0,strokeMiterLimit=0,strokeDashArray=None,strokeOpacity=None))
		self.add(Rect(95,50,2.857143,6.944444,rx=0,ry=0,fillColor=Color(1,0,0,1),fillOpacity=None,strokeColor=Color(1,0,0,1),strokeWidth=1,strokeLineCap=0,strokeLineJoin=0,strokeMiterLimit=0,strokeDashArray=None,strokeOpacity=None))
		self.add(Rect(97.85714,50,2.857143,6.944444,rx=0,ry=0,fillColor=Color(1,.090909,.090909,1),fillOpacity=None,strokeColor=Color(1,.090909,.090909,1),strokeWidth=1,strokeLineCap=0,strokeLineJoin=0,strokeMiterLimit=0,strokeDashArray=None,strokeOpacity=None))
		self.add(Rect(100.7143,50,2.857143,6.944444,rx=0,ry=0,fillColor=Color(1,.181818,.181818,1),fillOpacity=None,strokeColor=Color(1,.181818,.181818,1),strokeWidth=1,strokeLineCap=0,strokeLineJoin=0,strokeMiterLimit=0,strokeDashArray=None,strokeOpacity=None))
		self.add(Rect(103.5714,50,2.857143,6.944444,rx=0,ry=0,fillColor=Color(1,.272727,.272727,1),fillOpacity=None,strokeColor=Color(1,.272727,.272727,1),strokeWidth=1,strokeLineCap=0,strokeLineJoin=0,strokeMiterLimit=0,strokeDashArray=None,strokeOpacity=None))
		self.add(Rect(106.4286,50,2.857143,6.944444,rx=0,ry=0,fillColor=Color(1,.363636,.363636,1),fillOpacity=None,strokeColor=Color(1,.363636,.363636,1),strokeWidth=1,strokeLineCap=0,strokeLineJoin=0,strokeMiterLimit=0,strokeDashArray=None,strokeOpacity=None))
		self.add(Rect(109.2857,50,2.857143,6.944444,rx=0,ry=0,fillColor=Color(1,.454545,.454545,1),fillOpacity=None,strokeColor=Color(1,.454545,.454545,1),strokeWidth=1,strokeLineCap=0,strokeLineJoin=0,strokeMiterLimit=0,strokeDashArray=None,strokeOpacity=None))
		self.add(Rect(112.1429,50,2.857143,6.944444,rx=0,ry=0,fillColor=Color(1,.545455,.545455,1),fillOpacity=None,strokeColor=Color(1,.545455,.545455,1),strokeWidth=1,strokeLineCap=0,strokeLineJoin=0,strokeMiterLimit=0,strokeDashArray=None,strokeOpacity=None))
		self.add(Rect(115,50,2.857143,6.944444,rx=0,ry=0,fillColor=Color(1,.636364,.636364,1),fillOpacity=None,strokeColor=Color(1,.636364,.636364,1),strokeWidth=1,strokeLineCap=0,strokeLineJoin=0,strokeMiterLimit=0,strokeDashArray=None,strokeOpacity=None))
		self.add(Rect(117.8571,50,2.857143,6.944444,rx=0,ry=0,fillColor=Color(1,.727273,.727273,1),fillOpacity=None,strokeColor=Color(1,.727273,.727273,1),strokeWidth=1,strokeLineCap=0,strokeLineJoin=0,strokeMiterLimit=0,strokeDashArray=None,strokeOpacity=None))
		self.add(Rect(120.7143,50,2.857143,6.944444,rx=0,ry=0,fillColor=Color(1,.818182,.818182,1),fillOpacity=None,strokeColor=Color(1,.818182,.818182,1),strokeWidth=1,strokeLineCap=0,strokeLineJoin=0,strokeMiterLimit=0,strokeDashArray=None,strokeOpacity=None))
		self.add(Rect(123.5714,50,2.857143,6.944444,rx=0,ry=0,fillColor=Color(1,.909091,.909091,1),fillOpacity=None,strokeColor=Color(1,.909091,.909091,1),strokeWidth=1,strokeLineCap=0,strokeLineJoin=0,strokeMiterLimit=0,strokeDashArray=None,strokeOpacity=None))
		self.add(Rect(126.4286,50,2.857143,6.944444,rx=0,ry=0,fillColor=Color(1,1,1,1),fillOpacity=None,strokeColor=Color(1,1,1,1),strokeWidth=1,strokeLineCap=0,strokeLineJoin=0,strokeMiterLimit=0,strokeDashArray=None,strokeOpacity=None))
		self.add(Rect(129.2857,50,2.857143,6.944444,rx=0,ry=0,fillColor=Color(1,.888889,.888889,1),fillOpacity=None,strokeColor=Color(1,.888889,.888889,1),strokeWidth=1,strokeLineCap=0,strokeLineJoin=0,strokeMiterLimit=0,strokeDashArray=None,strokeOpacity=None))
		self.add(Rect(132.1429,50,2.857143,6.944444,rx=0,ry=0,fillColor=Color(1,.777778,.777778,1),fillOpacity=None,strokeColor=Color(1,.777778,.777778,1),strokeWidth=1,strokeLineCap=0,strokeLineJoin=0,strokeMiterLimit=0,strokeDashArray=None,strokeOpacity=None))
		self.add(Rect(135,50,2.857143,6.944444,rx=0,ry=0,fillColor=Color(1,.666667,.666667,1),fillOpacity=None,strokeColor=Color(1,.666667,.666667,1),strokeWidth=1,strokeLineCap=0,strokeLineJoin=0,strokeMiterLimit=0,strokeDashArray=None,strokeOpacity=None))
		self.add(Rect(137.8571,50,2.857143,6.944444,rx=0,ry=0,fillColor=Color(1,.555556,.555556,1),fillOpacity=None,strokeColor=Color(1,.555556,.555556,1),strokeWidth=1,strokeLineCap=0,strokeLineJoin=0,strokeMiterLimit=0,strokeDashArray=None,strokeOpacity=None))
		self.add(Rect(140.7143,50,2.857143,6.944444,rx=0,ry=0,fillColor=Color(1,.444444,.444444,1),fillOpacity=None,strokeColor=Color(1,.444444,.444444,1),strokeWidth=1,strokeLineCap=0,strokeLineJoin=0,strokeMiterLimit=0,strokeDashArray=None,strokeOpacity=None))
		self.add(Rect(143.5714,50,2.857143,6.944444,rx=0,ry=0,fillColor=Color(1,.333333,.333333,1),fillOpacity=None,strokeColor=Color(1,.333333,.333333,1),strokeWidth=1,strokeLineCap=0,strokeLineJoin=0,strokeMiterLimit=0,strokeDashArray=None,strokeOpacity=None))
		self.add(Rect(146.4286,50,2.857143,6.944444,rx=0,ry=0,fillColor=Color(1,.222222,.222222,1),fillOpacity=None,strokeColor=Color(1,.222222,.222222,1),strokeWidth=1,strokeLineCap=0,strokeLineJoin=0,strokeMiterLimit=0,strokeDashArray=None,strokeOpacity=None))
		self.add(Rect(149.2857,50,2.857143,6.944444,rx=0,ry=0,fillColor=Color(1,.111111,.111111,1),fillOpacity=None,strokeColor=Color(1,.111111,.111111,1),strokeWidth=1,strokeLineCap=0,strokeLineJoin=0,strokeMiterLimit=0,strokeDashArray=None,strokeOpacity=None))
		self.add(Rect(152.1429,50,2.857143,6.944444,rx=0,ry=0,fillColor=Color(1,0,0,1),fillOpacity=None,strokeColor=Color(1,0,0,1),strokeWidth=1,strokeLineCap=0,strokeLineJoin=0,strokeMiterLimit=0,strokeDashArray=None,strokeOpacity=None))
		self.add(Rect(95,50,60,6.944444,rx=0,ry=0,fillColor=None,fillOpacity=None,strokeColor=Color(0,0,0,1),strokeWidth=0,strokeLineCap=0,strokeLineJoin=0,strokeMiterLimit=0,strokeDashArray=None,strokeOpacity=None))
		self.add(Rect(245,50,2.857143,41.66667,rx=0,ry=0,fillColor=Color(1,0,0,1),fillOpacity=None,strokeColor=Color(1,0,0,1),strokeWidth=1,strokeLineCap=0,strokeLineJoin=0,strokeMiterLimit=0,strokeDashArray=None,strokeOpacity=None))
		self.add(Rect(247.8571,50,2.857143,41.66667,rx=0,ry=0,fillColor=Color(1,.090909,.090909,1),fillOpacity=None,strokeColor=Color(1,.090909,.090909,1),strokeWidth=1,strokeLineCap=0,strokeLineJoin=0,strokeMiterLimit=0,strokeDashArray=None,strokeOpacity=None))
		self.add(Rect(250.7143,50,2.857143,41.66667,rx=0,ry=0,fillColor=Color(1,.181818,.181818,1),fillOpacity=None,strokeColor=Color(1,.181818,.181818,1),strokeWidth=1,strokeLineCap=0,strokeLineJoin=0,strokeMiterLimit=0,strokeDashArray=None,strokeOpacity=None))
		self.add(Rect(253.5714,50,2.857143,41.66667,rx=0,ry=0,fillColor=Color(1,.272727,.272727,1),fillOpacity=None,strokeColor=Color(1,.272727,.272727,1),strokeWidth=1,strokeLineCap=0,strokeLineJoin=0,strokeMiterLimit=0,strokeDashArray=None,strokeOpacity=None))
		self.add(Rect(256.4286,50,2.857143,41.66667,rx=0,ry=0,fillColor=Color(1,.363636,.363636,1),fillOpacity=None,strokeColor=Color(1,.363636,.363636,1),strokeWidth=1,strokeLineCap=0,strokeLineJoin=0,strokeMiterLimit=0,strokeDashArray=None,strokeOpacity=None))
		self.add(Rect(259.2857,50,2.857143,41.66667,rx=0,ry=0,fillColor=Color(1,.454545,.454545,1),fillOpacity=None,strokeColor=Color(1,.454545,.454545,1),strokeWidth=1,strokeLineCap=0,strokeLineJoin=0,strokeMiterLimit=0,strokeDashArray=None,strokeOpacity=None))
		self.add(Rect(262.1429,50,2.857143,41.66667,rx=0,ry=0,fillColor=Color(1,.545455,.545455,1),fillOpacity=None,strokeColor=Color(1,.545455,.545455,1),strokeWidth=1,strokeLineCap=0,strokeLineJoin=0,strokeMiterLimit=0,strokeDashArray=None,strokeOpacity=None))
		self.add(Rect(265,50,2.857143,41.66667,rx=0,ry=0,fillColor=Color(1,.636364,.636364,1),fillOpacity=None,strokeColor=Color(1,.636364,.636364,1),strokeWidth=1,strokeLineCap=0,strokeLineJoin=0,strokeMiterLimit=0,strokeDashArray=None,strokeOpacity=None))
		self.add(Rect(267.8571,50,2.857143,41.66667,rx=0,ry=0,fillColor=Color(1,.727273,.727273,1),fillOpacity=None,strokeColor=Color(1,.727273,.727273,1),strokeWidth=1,strokeLineCap=0,strokeLineJoin=0,strokeMiterLimit=0,strokeDashArray=None,strokeOpacity=None))
		self.add(Rect(270.7143,50,2.857143,41.66667,rx=0,ry=0,fillColor=Color(1,.818182,.818182,1),fillOpacity=None,strokeColor=Color(1,.818182,.818182,1),strokeWidth=1,strokeLineCap=0,strokeLineJoin=0,strokeMiterLimit=0,strokeDashArray=None,strokeOpacity=None))
		self.add(Rect(273.5714,50,2.857143,41.66667,rx=0,ry=0,fillColor=Color(1,.909091,.909091,1),fillOpacity=None,strokeColor=Color(1,.909091,.909091,1),strokeWidth=1,strokeLineCap=0,strokeLineJoin=0,strokeMiterLimit=0,strokeDashArray=None,strokeOpacity=None))
		self.add(Rect(276.4286,50,2.857143,41.66667,rx=0,ry=0,fillColor=Color(1,1,1,1),fillOpacity=None,strokeColor=Color(1,1,1,1),strokeWidth=1,strokeLineCap=0,strokeLineJoin=0,strokeMiterLimit=0,strokeDashArray=None,strokeOpacity=None))
		self.add(Rect(279.2857,50,2.857143,41.66667,rx=0,ry=0,fillColor=Color(1,.888889,.888889,1),fillOpacity=None,strokeColor=Color(1,.888889,.888889,1),strokeWidth=1,strokeLineCap=0,strokeLineJoin=0,strokeMiterLimit=0,strokeDashArray=None,strokeOpacity=None))
		self.add(Rect(282.1429,50,2.857143,41.66667,rx=0,ry=0,fillColor=Color(1,.777778,.777778,1),fillOpacity=None,strokeColor=Color(1,.777778,.777778,1),strokeWidth=1,strokeLineCap=0,strokeLineJoin=0,strokeMiterLimit=0,strokeDashArray=None,strokeOpacity=None))
		self.add(Rect(285,50,2.857143,41.66667,rx=0,ry=0,fillColor=Color(1,.666667,.666667,1),fillOpacity=None,strokeColor=Color(1,.666667,.666667,1),strokeWidth=1,strokeLineCap=0,strokeLineJoin=0,strokeMiterLimit=0,strokeDashArray=None,strokeOpacity=None))
		self.add(Rect(287.8571,50,2.857143,41.66667,rx=0,ry=0,fillColor=Color(1,.555556,.555556,1),fillOpacity=None,strokeColor=Color(1,.555556,.555556,1),strokeWidth=1,strokeLineCap=0,strokeLineJoin=0,strokeMiterLimit=0,strokeDashArray=None,strokeOpacity=None))
		self.add(Rect(290.7143,50,2.857143,41.66667,rx=0,ry=0,fillColor=Color(1,.444444,.444444,1),fillOpacity=None,strokeColor=Color(1,.444444,.444444,1),strokeWidth=1,strokeLineCap=0,strokeLineJoin=0,strokeMiterLimit=0,strokeDashArray=None,strokeOpacity=None))
		self.add(Rect(293.5714,50,2.857143,41.66667,rx=0,ry=0,fillColor=Color(1,.333333,.333333,1),fillOpacity=None,strokeColor=Color(1,.333333,.333333,1),strokeWidth=1,strokeLineCap=0,strokeLineJoin=0,strokeMiterLimit=0,strokeDashArray=None,strokeOpacity=None))
		self.add(Rect(296.4286,50,2.857143,41.66667,rx=0,ry=0,fillColor=Color(1,.222222,.222222,1),fillOpacity=None,strokeColor=Color(1,.222222,.222222,1),strokeWidth=1,strokeLineCap=0,strokeLineJoin=0,strokeMiterLimit=0,strokeDashArray=None,strokeOpacity=None))
		self.add(Rect(299.2857,50,2.857143,41.66667,rx=0,ry=0,fillColor=Color(1,.111111,.111111,1),fillOpacity=None,strokeColor=Color(1,.111111,.111111,1),strokeWidth=1,strokeLineCap=0,strokeLineJoin=0,strokeMiterLimit=0,strokeDashArray=None,strokeOpacity=None))
		self.add(Rect(302.1429,50,2.857143,41.66667,rx=0,ry=0,fillColor=Color(1,0,0,1),fillOpacity=None,strokeColor=Color(1,0,0,1),strokeWidth=1,strokeLineCap=0,strokeLineJoin=0,strokeMiterLimit=0,strokeDashArray=None,strokeOpacity=None))
		self.add(Rect(245,50,60,41.66667,rx=0,ry=0,fillColor=None,fillOpacity=None,strokeColor=Color(0,0,0,1),strokeWidth=0,strokeLineCap=0,strokeLineJoin=0,strokeMiterLimit=0,strokeDashArray=None,strokeOpacity=None))
		self.add(Rect(95,56.94444,2.857143,13.88889,rx=0,ry=0,fillColor=Color(1,0,1,1),fillOpacity=None,strokeColor=Color(1,0,1,1),strokeWidth=1,strokeLineCap=0,strokeLineJoin=0,strokeMiterLimit=0,strokeDashArray=None,strokeOpacity=None))
		self.add(Rect(97.85714,56.94444,2.857143,13.88889,rx=0,ry=0,fillColor=Color(1,.090909,1,1),fillOpacity=None,strokeColor=Color(1,.090909,1,1),strokeWidth=1,strokeLineCap=0,strokeLineJoin=0,strokeMiterLimit=0,strokeDashArray=None,strokeOpacity=None))
		self.add(Rect(100.7143,56.94444,2.857143,13.88889,rx=0,ry=0,fillColor=Color(1,.181818,1,1),fillOpacity=None,strokeColor=Color(1,.181818,1,1),strokeWidth=1,strokeLineCap=0,strokeLineJoin=0,strokeMiterLimit=0,strokeDashArray=None,strokeOpacity=None))
		self.add(Rect(103.5714,56.94444,2.857143,13.88889,rx=0,ry=0,fillColor=Color(1,.272727,1,1),fillOpacity=None,strokeColor=Color(1,.272727,1,1),strokeWidth=1,strokeLineCap=0,strokeLineJoin=0,strokeMiterLimit=0,strokeDashArray=None,strokeOpacity=None))
		self.add(Rect(106.4286,56.94444,2.857143,13.88889,rx=0,ry=0,fillColor=Color(1,.363636,1,1),fillOpacity=None,strokeColor=Color(1,.363636,1,1),strokeWidth=1,strokeLineCap=0,strokeLineJoin=0,strokeMiterLimit=0,strokeDashArray=None,strokeOpacity=None))
		self.add(Rect(109.2857,56.94444,2.857143,13.88889,rx=0,ry=0,fillColor=Color(1,.454545,1,1),fillOpacity=None,strokeColor=Color(1,.454545,1,1),strokeWidth=1,strokeLineCap=0,strokeLineJoin=0,strokeMiterLimit=0,strokeDashArray=None,strokeOpacity=None))
		self.add(Rect(112.1429,56.94444,2.857143,13.88889,rx=0,ry=0,fillColor=Color(1,.545455,1,1),fillOpacity=None,strokeColor=Color(1,.545455,1,1),strokeWidth=1,strokeLineCap=0,strokeLineJoin=0,strokeMiterLimit=0,strokeDashArray=None,strokeOpacity=None))
		self.add(Rect(115,56.94444,2.857143,13.88889,rx=0,ry=0,fillColor=Color(1,.636364,1,1),fillOpacity=None,strokeColor=Color(1,.636364,1,1),strokeWidth=1,strokeLineCap=0,strokeLineJoin=0,strokeMiterLimit=0,strokeDashArray=None,strokeOpacity=None))
		self.add(Rect(117.8571,56.94444,2.857143,13.88889,rx=0,ry=0,fillColor=Color(1,.727273,1,1),fillOpacity=None,strokeColor=Color(1,.727273,1,1),strokeWidth=1,strokeLineCap=0,strokeLineJoin=0,strokeMiterLimit=0,strokeDashArray=None,strokeOpacity=None))
		self.add(Rect(120.7143,56.94444,2.857143,13.88889,rx=0,ry=0,fillColor=Color(1,.818182,1,1),fillOpacity=None,strokeColor=Color(1,.818182,1,1),strokeWidth=1,strokeLineCap=0,strokeLineJoin=0,strokeMiterLimit=0,strokeDashArray=None,strokeOpacity=None))
		self.add(Rect(123.5714,56.94444,2.857143,13.88889,rx=0,ry=0,fillColor=Color(1,.909091,1,1),fillOpacity=None,strokeColor=Color(1,.909091,1,1),strokeWidth=1,strokeLineCap=0,strokeLineJoin=0,strokeMiterLimit=0,strokeDashArray=None,strokeOpacity=None))
		self.add(Rect(126.4286,56.94444,2.857143,13.88889,rx=0,ry=0,fillColor=Color(1,1,1,1),fillOpacity=None,strokeColor=Color(1,1,1,1),strokeWidth=1,strokeLineCap=0,strokeLineJoin=0,strokeMiterLimit=0,strokeDashArray=None,strokeOpacity=None))
		self.add(Rect(129.2857,56.94444,2.857143,13.88889,rx=0,ry=0,fillColor=Color(1,.888889,1,1),fillOpacity=None,strokeColor=Color(1,.888889,1,1),strokeWidth=1,strokeLineCap=0,strokeLineJoin=0,strokeMiterLimit=0,strokeDashArray=None,strokeOpacity=None))
		self.add(Rect(132.1429,56.94444,2.857143,13.88889,rx=0,ry=0,fillColor=Color(1,.777778,1,1),fillOpacity=None,strokeColor=Color(1,.777778,1,1),strokeWidth=1,strokeLineCap=0,strokeLineJoin=0,strokeMiterLimit=0,strokeDashArray=None,strokeOpacity=None))
		self.add(Rect(135,56.94444,2.857143,13.88889,rx=0,ry=0,fillColor=Color(1,.666667,1,1),fillOpacity=None,strokeColor=Color(1,.666667,1,1),strokeWidth=1,strokeLineCap=0,strokeLineJoin=0,strokeMiterLimit=0,strokeDashArray=None,strokeOpacity=None))
		self.add(Rect(137.8571,56.94444,2.857143,13.88889,rx=0,ry=0,fillColor=Color(1,.555556,1,1),fillOpacity=None,strokeColor=Color(1,.555556,1,1),strokeWidth=1,strokeLineCap=0,strokeLineJoin=0,strokeMiterLimit=0,strokeDashArray=None,strokeOpacity=None))
		self.add(Rect(140.7143,56.94444,2.857143,13.88889,rx=0,ry=0,fillColor=Color(1,.444444,1,1),fillOpacity=None,strokeColor=Color(1,.444444,1,1),strokeWidth=1,strokeLineCap=0,strokeLineJoin=0,strokeMiterLimit=0,strokeDashArray=None,strokeOpacity=None))
		self.add(Rect(143.5714,56.94444,2.857143,13.88889,rx=0,ry=0,fillColor=Color(1,.333333,1,1),fillOpacity=None,strokeColor=Color(1,.333333,1,1),strokeWidth=1,strokeLineCap=0,strokeLineJoin=0,strokeMiterLimit=0,strokeDashArray=None,strokeOpacity=None))
		self.add(Rect(146.4286,56.94444,2.857143,13.88889,rx=0,ry=0,fillColor=Color(1,.222222,1,1),fillOpacity=None,strokeColor=Color(1,.222222,1,1),strokeWidth=1,strokeLineCap=0,strokeLineJoin=0,strokeMiterLimit=0,strokeDashArray=None,strokeOpacity=None))
		self.add(Rect(149.2857,56.94444,2.857143,13.88889,rx=0,ry=0,fillColor=Color(1,.111111,1,1),fillOpacity=None,strokeColor=Color(1,.111111,1,1),strokeWidth=1,strokeLineCap=0,strokeLineJoin=0,strokeMiterLimit=0,strokeDashArray=None,strokeOpacity=None))
		self.add(Rect(152.1429,56.94444,2.857143,13.88889,rx=0,ry=0,fillColor=Color(1,0,1,1),fillOpacity=None,strokeColor=Color(1,0,1,1),strokeWidth=1,strokeLineCap=0,strokeLineJoin=0,strokeMiterLimit=0,strokeDashArray=None,strokeOpacity=None))
		self.add(Rect(95,56.94444,60,13.88889,rx=0,ry=0,fillColor=None,fillOpacity=None,strokeColor=Color(0,0,0,1),strokeWidth=0,strokeLineCap=0,strokeLineJoin=0,strokeMiterLimit=0,strokeDashArray=None,strokeOpacity=None))
		self.add(Rect(245,91.66667,2.857143,34.72222,rx=0,ry=0,fillColor=Color(1,0,1,1),fillOpacity=None,strokeColor=Color(1,0,1,1),strokeWidth=1,strokeLineCap=0,strokeLineJoin=0,strokeMiterLimit=0,strokeDashArray=None,strokeOpacity=None))
		self.add(Rect(247.8571,91.66667,2.857143,34.72222,rx=0,ry=0,fillColor=Color(1,.090909,1,1),fillOpacity=None,strokeColor=Color(1,.090909,1,1),strokeWidth=1,strokeLineCap=0,strokeLineJoin=0,strokeMiterLimit=0,strokeDashArray=None,strokeOpacity=None))
		self.add(Rect(250.7143,91.66667,2.857143,34.72222,rx=0,ry=0,fillColor=Color(1,.181818,1,1),fillOpacity=None,strokeColor=Color(1,.181818,1,1),strokeWidth=1,strokeLineCap=0,strokeLineJoin=0,strokeMiterLimit=0,strokeDashArray=None,strokeOpacity=None))
		self.add(Rect(253.5714,91.66667,2.857143,34.72222,rx=0,ry=0,fillColor=Color(1,.272727,1,1),fillOpacity=None,strokeColor=Color(1,.272727,1,1),strokeWidth=1,strokeLineCap=0,strokeLineJoin=0,strokeMiterLimit=0,strokeDashArray=None,strokeOpacity=None))
		self.add(Rect(256.4286,91.66667,2.857143,34.72222,rx=0,ry=0,fillColor=Color(1,.363636,1,1),fillOpacity=None,strokeColor=Color(1,.363636,1,1),strokeWidth=1,strokeLineCap=0,strokeLineJoin=0,strokeMiterLimit=0,strokeDashArray=None,strokeOpacity=None))
		self.add(Rect(259.2857,91.66667,2.857143,34.72222,rx=0,ry=0,fillColor=Color(1,.454545,1,1),fillOpacity=None,strokeColor=Color(1,.454545,1,1),strokeWidth=1,strokeLineCap=0,strokeLineJoin=0,strokeMiterLimit=0,strokeDashArray=None,strokeOpacity=None))
		self.add(Rect(262.1429,91.66667,2.857143,34.72222,rx=0,ry=0,fillColor=Color(1,.545455,1,1),fillOpacity=None,strokeColor=Color(1,.545455,1,1),strokeWidth=1,strokeLineCap=0,strokeLineJoin=0,strokeMiterLimit=0,strokeDashArray=None,strokeOpacity=None))
		self.add(Rect(265,91.66667,2.857143,34.72222,rx=0,ry=0,fillColor=Color(1,.636364,1,1),fillOpacity=None,strokeColor=Color(1,.636364,1,1),strokeWidth=1,strokeLineCap=0,strokeLineJoin=0,strokeMiterLimit=0,strokeDashArray=None,strokeOpacity=None))
		self.add(Rect(267.8571,91.66667,2.857143,34.72222,rx=0,ry=0,fillColor=Color(1,.727273,1,1),fillOpacity=None,strokeColor=Color(1,.727273,1,1),strokeWidth=1,strokeLineCap=0,strokeLineJoin=0,strokeMiterLimit=0,strokeDashArray=None,strokeOpacity=None))
		self.add(Rect(270.7143,91.66667,2.857143,34.72222,rx=0,ry=0,fillColor=Color(1,.818182,1,1),fillOpacity=None,strokeColor=Color(1,.818182,1,1),strokeWidth=1,strokeLineCap=0,strokeLineJoin=0,strokeMiterLimit=0,strokeDashArray=None,strokeOpacity=None))
		self.add(Rect(273.5714,91.66667,2.857143,34.72222,rx=0,ry=0,fillColor=Color(1,.909091,1,1),fillOpacity=None,strokeColor=Color(1,.909091,1,1),strokeWidth=1,strokeLineCap=0,strokeLineJoin=0,strokeMiterLimit=0,strokeDashArray=None,strokeOpacity=None))
		self.add(Rect(276.4286,91.66667,2.857143,34.72222,rx=0,ry=0,fillColor=Color(1,1,1,1),fillOpacity=None,strokeColor=Color(1,1,1,1),strokeWidth=1,strokeLineCap=0,strokeLineJoin=0,strokeMiterLimit=0,strokeDashArray=None,strokeOpacity=None))
		self.add(Rect(279.2857,91.66667,2.857143,34.72222,rx=0,ry=0,fillColor=Color(1,.888889,1,1),fillOpacity=None,strokeColor=Color(1,.888889,1,1),strokeWidth=1,strokeLineCap=0,strokeLineJoin=0,strokeMiterLimit=0,strokeDashArray=None,strokeOpacity=None))
		self.add(Rect(282.1429,91.66667,2.857143,34.72222,rx=0,ry=0,fillColor=Color(1,.777778,1,1),fillOpacity=None,strokeColor=Color(1,.777778,1,1),strokeWidth=1,strokeLineCap=0,strokeLineJoin=0,strokeMiterLimit=0,strokeDashArray=None,strokeOpacity=None))
		self.add(Rect(285,91.66667,2.857143,34.72222,rx=0,ry=0,fillColor=Color(1,.666667,1,1),fillOpacity=None,strokeColor=Color(1,.666667,1,1),strokeWidth=1,strokeLineCap=0,strokeLineJoin=0,strokeMiterLimit=0,strokeDashArray=None,strokeOpacity=None))
		self.add(Rect(287.8571,91.66667,2.857143,34.72222,rx=0,ry=0,fillColor=Color(1,.555556,1,1),fillOpacity=None,strokeColor=Color(1,.555556,1,1),strokeWidth=1,strokeLineCap=0,strokeLineJoin=0,strokeMiterLimit=0,strokeDashArray=None,strokeOpacity=None))
		self.add(Rect(290.7143,91.66667,2.857143,34.72222,rx=0,ry=0,fillColor=Color(1,.444444,1,1),fillOpacity=None,strokeColor=Color(1,.444444,1,1),strokeWidth=1,strokeLineCap=0,strokeLineJoin=0,strokeMiterLimit=0,strokeDashArray=None,strokeOpacity=None))
		self.add(Rect(293.5714,91.66667,2.857143,34.72222,rx=0,ry=0,fillColor=Color(1,.333333,1,1),fillOpacity=None,strokeColor=Color(1,.333333,1,1),strokeWidth=1,strokeLineCap=0,strokeLineJoin=0,strokeMiterLimit=0,strokeDashArray=None,strokeOpacity=None))
		self.add(Rect(296.4286,91.66667,2.857143,34.72222,rx=0,ry=0,fillColor=Color(1,.222222,1,1),fillOpacity=None,strokeColor=Color(1,.222222,1,1),strokeWidth=1,strokeLineCap=0,strokeLineJoin=0,strokeMiterLimit=0,strokeDashArray=None,strokeOpacity=None))
		self.add(Rect(299.2857,91.66667,2.857143,34.72222,rx=0,ry=0,fillColor=Color(1,.111111,1,1),fillOpacity=None,strokeColor=Color(1,.111111,1,1),strokeWidth=1,strokeLineCap=0,strokeLineJoin=0,strokeMiterLimit=0,strokeDashArray=None,strokeOpacity=None))
		self.add(Rect(302.1429,91.66667,2.857143,34.72222,rx=0,ry=0,fillColor=Color(1,0,1,1),fillOpacity=None,strokeColor=Color(1,0,1,1),strokeWidth=1,strokeLineCap=0,strokeLineJoin=0,strokeMiterLimit=0,strokeDashArray=None,strokeOpacity=None))
		self.add(Rect(245,91.66667,60,34.72222,rx=0,ry=0,fillColor=None,fillOpacity=None,strokeColor=Color(0,0,0,1),strokeWidth=0,strokeLineCap=0,strokeLineJoin=0,strokeMiterLimit=0,strokeDashArray=None,strokeOpacity=None))
		self.add(Rect(95,70.83333,2.857143,20.83333,rx=0,ry=0,fillColor=Color(0,.501961,0,1),fillOpacity=None,strokeColor=Color(0,.501961,0,1),strokeWidth=1,strokeLineCap=0,strokeLineJoin=0,strokeMiterLimit=0,strokeDashArray=None,strokeOpacity=None))
		self.add(Rect(97.85714,70.83333,2.857143,20.83333,rx=0,ry=0,fillColor=Color(.090909,.547237,.090909,1),fillOpacity=None,strokeColor=Color(.090909,.547237,.090909,1),strokeWidth=1,strokeLineCap=0,strokeLineJoin=0,strokeMiterLimit=0,strokeDashArray=None,strokeOpacity=None))
		self.add(Rect(100.7143,70.83333,2.857143,20.83333,rx=0,ry=0,fillColor=Color(.181818,.592513,.181818,1),fillOpacity=None,strokeColor=Color(.181818,.592513,.181818,1),strokeWidth=1,strokeLineCap=0,strokeLineJoin=0,strokeMiterLimit=0,strokeDashArray=None,strokeOpacity=None))
		self.add(Rect(103.5714,70.83333,2.857143,20.83333,rx=0,ry=0,fillColor=Color(.272727,.63779,.272727,1),fillOpacity=None,strokeColor=Color(.272727,.63779,.272727,1),strokeWidth=1,strokeLineCap=0,strokeLineJoin=0,strokeMiterLimit=0,strokeDashArray=None,strokeOpacity=None))
		self.add(Rect(106.4286,70.83333,2.857143,20.83333,rx=0,ry=0,fillColor=Color(.363636,.683066,.363636,1),fillOpacity=None,strokeColor=Color(.363636,.683066,.363636,1),strokeWidth=1,strokeLineCap=0,strokeLineJoin=0,strokeMiterLimit=0,strokeDashArray=None,strokeOpacity=None))
		self.add(Rect(109.2857,70.83333,2.857143,20.83333,rx=0,ry=0,fillColor=Color(.454545,.728342,.454545,1),fillOpacity=None,strokeColor=Color(.454545,.728342,.454545,1),strokeWidth=1,strokeLineCap=0,strokeLineJoin=0,strokeMiterLimit=0,strokeDashArray=None,strokeOpacity=None))
		self.add(Rect(112.1429,70.83333,2.857143,20.83333,rx=0,ry=0,fillColor=Color(.545455,.773619,.545455,1),fillOpacity=None,strokeColor=Color(.545455,.773619,.545455,1),strokeWidth=1,strokeLineCap=0,strokeLineJoin=0,strokeMiterLimit=0,strokeDashArray=None,strokeOpacity=None))
		self.add(Rect(115,70.83333,2.857143,20.83333,rx=0,ry=0,fillColor=Color(.636364,.818895,.636364,1),fillOpacity=None,strokeColor=Color(.636364,.818895,.636364,1),strokeWidth=1,strokeLineCap=0,strokeLineJoin=0,strokeMiterLimit=0,strokeDashArray=None,strokeOpacity=None))
		self.add(Rect(117.8571,70.83333,2.857143,20.83333,rx=0,ry=0,fillColor=Color(.727273,.864171,.727273,1),fillOpacity=None,strokeColor=Color(.727273,.864171,.727273,1),strokeWidth=1,strokeLineCap=0,strokeLineJoin=0,strokeMiterLimit=0,strokeDashArray=None,strokeOpacity=None))
		self.add(Rect(120.7143,70.83333,2.857143,20.83333,rx=0,ry=0,fillColor=Color(.818182,.909447,.818182,1),fillOpacity=None,strokeColor=Color(.818182,.909447,.818182,1),strokeWidth=1,strokeLineCap=0,strokeLineJoin=0,strokeMiterLimit=0,strokeDashArray=None,strokeOpacity=None))
		self.add(Rect(123.5714,70.83333,2.857143,20.83333,rx=0,ry=0,fillColor=Color(.909091,.954724,.909091,1),fillOpacity=None,strokeColor=Color(.909091,.954724,.909091,1),strokeWidth=1,strokeLineCap=0,strokeLineJoin=0,strokeMiterLimit=0,strokeDashArray=None,strokeOpacity=None))
		self.add(Rect(126.4286,70.83333,2.857143,20.83333,rx=0,ry=0,fillColor=Color(1,1,1,1),fillOpacity=None,strokeColor=Color(1,1,1,1),strokeWidth=1,strokeLineCap=0,strokeLineJoin=0,strokeMiterLimit=0,strokeDashArray=None,strokeOpacity=None))
		self.add(Rect(129.2857,70.83333,2.857143,20.83333,rx=0,ry=0,fillColor=Color(.888889,.944662,.888889,1),fillOpacity=None,strokeColor=Color(.888889,.944662,.888889,1),strokeWidth=1,strokeLineCap=0,strokeLineJoin=0,strokeMiterLimit=0,strokeDashArray=None,strokeOpacity=None))
		self.add(Rect(132.1429,70.83333,2.857143,20.83333,rx=0,ry=0,fillColor=Color(.777778,.889325,.777778,1),fillOpacity=None,strokeColor=Color(.777778,.889325,.777778,1),strokeWidth=1,strokeLineCap=0,strokeLineJoin=0,strokeMiterLimit=0,strokeDashArray=None,strokeOpacity=None))
		self.add(Rect(135,70.83333,2.857143,20.83333,rx=0,ry=0,fillColor=Color(.666667,.833987,.666667,1),fillOpacity=None,strokeColor=Color(.666667,.833987,.666667,1),strokeWidth=1,strokeLineCap=0,strokeLineJoin=0,strokeMiterLimit=0,strokeDashArray=None,strokeOpacity=None))
		self.add(Rect(137.8571,70.83333,2.857143,20.83333,rx=0,ry=0,fillColor=Color(.555556,.778649,.555556,1),fillOpacity=None,strokeColor=Color(.555556,.778649,.555556,1),strokeWidth=1,strokeLineCap=0,strokeLineJoin=0,strokeMiterLimit=0,strokeDashArray=None,strokeOpacity=None))
		self.add(Rect(140.7143,70.83333,2.857143,20.83333,rx=0,ry=0,fillColor=Color(.444444,.723312,.444444,1),fillOpacity=None,strokeColor=Color(.444444,.723312,.444444,1),strokeWidth=1,strokeLineCap=0,strokeLineJoin=0,strokeMiterLimit=0,strokeDashArray=None,strokeOpacity=None))
		self.add(Rect(143.5714,70.83333,2.857143,20.83333,rx=0,ry=0,fillColor=Color(.333333,.667974,.333333,1),fillOpacity=None,strokeColor=Color(.333333,.667974,.333333,1),strokeWidth=1,strokeLineCap=0,strokeLineJoin=0,strokeMiterLimit=0,strokeDashArray=None,strokeOpacity=None))
		self.add(Rect(146.4286,70.83333,2.857143,20.83333,rx=0,ry=0,fillColor=Color(.222222,.612636,.222222,1),fillOpacity=None,strokeColor=Color(.222222,.612636,.222222,1),strokeWidth=1,strokeLineCap=0,strokeLineJoin=0,strokeMiterLimit=0,strokeDashArray=None,strokeOpacity=None))
		self.add(Rect(149.2857,70.83333,2.857143,20.83333,rx=0,ry=0,fillColor=Color(.111111,.557298,.111111,1),fillOpacity=None,strokeColor=Color(.111111,.557298,.111111,1),strokeWidth=1,strokeLineCap=0,strokeLineJoin=0,strokeMiterLimit=0,strokeDashArray=None,strokeOpacity=None))
		self.add(Rect(152.1429,70.83333,2.857143,20.83333,rx=0,ry=0,fillColor=Color(0,.501961,0,1),fillOpacity=None,strokeColor=Color(0,.501961,0,1),strokeWidth=1,strokeLineCap=0,strokeLineJoin=0,strokeMiterLimit=0,strokeDashArray=None,strokeOpacity=None))
		self.add(Rect(95,70.83333,60,20.83333,rx=0,ry=0,fillColor=None,fillOpacity=None,strokeColor=Color(0,0,0,1),strokeWidth=0,strokeLineCap=0,strokeLineJoin=0,strokeMiterLimit=0,strokeDashArray=None,strokeOpacity=None))
		self.add(Rect(245,126.3889,2.857143,27.77778,rx=0,ry=0,fillColor=Color(0,.501961,0,1),fillOpacity=None,strokeColor=Color(0,.501961,0,1),strokeWidth=1,strokeLineCap=0,strokeLineJoin=0,strokeMiterLimit=0,strokeDashArray=None,strokeOpacity=None))
		self.add(Rect(247.8571,126.3889,2.857143,27.77778,rx=0,ry=0,fillColor=Color(.090909,.547237,.090909,1),fillOpacity=None,strokeColor=Color(.090909,.547237,.090909,1),strokeWidth=1,strokeLineCap=0,strokeLineJoin=0,strokeMiterLimit=0,strokeDashArray=None,strokeOpacity=None))
		self.add(Rect(250.7143,126.3889,2.857143,27.77778,rx=0,ry=0,fillColor=Color(.181818,.592513,.181818,1),fillOpacity=None,strokeColor=Color(.181818,.592513,.181818,1),strokeWidth=1,strokeLineCap=0,strokeLineJoin=0,strokeMiterLimit=0,strokeDashArray=None,strokeOpacity=None))
		self.add(Rect(253.5714,126.3889,2.857143,27.77778,rx=0,ry=0,fillColor=Color(.272727,.63779,.272727,1),fillOpacity=None,strokeColor=Color(.272727,.63779,.272727,1),strokeWidth=1,strokeLineCap=0,strokeLineJoin=0,strokeMiterLimit=0,strokeDashArray=None,strokeOpacity=None))
		self.add(Rect(256.4286,126.3889,2.857143,27.77778,rx=0,ry=0,fillColor=Color(.363636,.683066,.363636,1),fillOpacity=None,strokeColor=Color(.363636,.683066,.363636,1),strokeWidth=1,strokeLineCap=0,strokeLineJoin=0,strokeMiterLimit=0,strokeDashArray=None,strokeOpacity=None))
		self.add(Rect(259.2857,126.3889,2.857143,27.77778,rx=0,ry=0,fillColor=Color(.454545,.728342,.454545,1),fillOpacity=None,strokeColor=Color(.454545,.728342,.454545,1),strokeWidth=1,strokeLineCap=0,strokeLineJoin=0,strokeMiterLimit=0,strokeDashArray=None,strokeOpacity=None))
		self.add(Rect(262.1429,126.3889,2.857143,27.77778,rx=0,ry=0,fillColor=Color(.545455,.773619,.545455,1),fillOpacity=None,strokeColor=Color(.545455,.773619,.545455,1),strokeWidth=1,strokeLineCap=0,strokeLineJoin=0,strokeMiterLimit=0,strokeDashArray=None,strokeOpacity=None))
		self.add(Rect(265,126.3889,2.857143,27.77778,rx=0,ry=0,fillColor=Color(.636364,.818895,.636364,1),fillOpacity=None,strokeColor=Color(.636364,.818895,.636364,1),strokeWidth=1,strokeLineCap=0,strokeLineJoin=0,strokeMiterLimit=0,strokeDashArray=None,strokeOpacity=None))
		self.add(Rect(267.8571,126.3889,2.857143,27.77778,rx=0,ry=0,fillColor=Color(.727273,.864171,.727273,1),fillOpacity=None,strokeColor=Color(.727273,.864171,.727273,1),strokeWidth=1,strokeLineCap=0,strokeLineJoin=0,strokeMiterLimit=0,strokeDashArray=None,strokeOpacity=None))
		self.add(Rect(270.7143,126.3889,2.857143,27.77778,rx=0,ry=0,fillColor=Color(.818182,.909447,.818182,1),fillOpacity=None,strokeColor=Color(.818182,.909447,.818182,1),strokeWidth=1,strokeLineCap=0,strokeLineJoin=0,strokeMiterLimit=0,strokeDashArray=None,strokeOpacity=None))
		self.add(Rect(273.5714,126.3889,2.857143,27.77778,rx=0,ry=0,fillColor=Color(.909091,.954724,.909091,1),fillOpacity=None,strokeColor=Color(.909091,.954724,.909091,1),strokeWidth=1,strokeLineCap=0,strokeLineJoin=0,strokeMiterLimit=0,strokeDashArray=None,strokeOpacity=None))
		self.add(Rect(276.4286,126.3889,2.857143,27.77778,rx=0,ry=0,fillColor=Color(1,1,1,1),fillOpacity=None,strokeColor=Color(1,1,1,1),strokeWidth=1,strokeLineCap=0,strokeLineJoin=0,strokeMiterLimit=0,strokeDashArray=None,strokeOpacity=None))
		self.add(Rect(279.2857,126.3889,2.857143,27.77778,rx=0,ry=0,fillColor=Color(.888889,.944662,.888889,1),fillOpacity=None,strokeColor=Color(.888889,.944662,.888889,1),strokeWidth=1,strokeLineCap=0,strokeLineJoin=0,strokeMiterLimit=0,strokeDashArray=None,strokeOpacity=None))
		self.add(Rect(282.1429,126.3889,2.857143,27.77778,rx=0,ry=0,fillColor=Color(.777778,.889325,.777778,1),fillOpacity=None,strokeColor=Color(.777778,.889325,.777778,1),strokeWidth=1,strokeLineCap=0,strokeLineJoin=0,strokeMiterLimit=0,strokeDashArray=None,strokeOpacity=None))
		self.add(Rect(285,126.3889,2.857143,27.77778,rx=0,ry=0,fillColor=Color(.666667,.833987,.666667,1),fillOpacity=None,strokeColor=Color(.666667,.833987,.666667,1),strokeWidth=1,strokeLineCap=0,strokeLineJoin=0,strokeMiterLimit=0,strokeDashArray=None,strokeOpacity=None))
		self.add(Rect(287.8571,126.3889,2.857143,27.77778,rx=0,ry=0,fillColor=Color(.555556,.778649,.555556,1),fillOpacity=None,strokeColor=Color(.555556,.778649,.555556,1),strokeWidth=1,strokeLineCap=0,strokeLineJoin=0,strokeMiterLimit=0,strokeDashArray=None,strokeOpacity=None))
		self.add(Rect(290.7143,126.3889,2.857143,27.77778,rx=0,ry=0,fillColor=Color(.444444,.723312,.444444,1),fillOpacity=None,strokeColor=Color(.444444,.723312,.444444,1),strokeWidth=1,strokeLineCap=0,strokeLineJoin=0,strokeMiterLimit=0,strokeDashArray=None,strokeOpacity=None))
		self.add(Rect(293.5714,126.3889,2.857143,27.77778,rx=0,ry=0,fillColor=Color(.333333,.667974,.333333,1),fillOpacity=None,strokeColor=Color(.333333,.667974,.333333,1),strokeWidth=1,strokeLineCap=0,strokeLineJoin=0,strokeMiterLimit=0,strokeDashArray=None,strokeOpacity=None))
		self.add(Rect(296.4286,126.3889,2.857143,27.77778,rx=0,ry=0,fillColor=Color(.222222,.612636,.222222,1),fillOpacity=None,strokeColor=Color(.222222,.612636,.222222,1),strokeWidth=1,strokeLineCap=0,strokeLineJoin=0,strokeMiterLimit=0,strokeDashArray=None,strokeOpacity=None))
		self.add(Rect(299.2857,126.3889,2.857143,27.77778,rx=0,ry=0,fillColor=Color(.111111,.557298,.111111,1),fillOpacity=None,strokeColor=Color(.111111,.557298,.111111,1),strokeWidth=1,strokeLineCap=0,strokeLineJoin=0,strokeMiterLimit=0,strokeDashArray=None,strokeOpacity=None))
		self.add(Rect(302.1429,126.3889,2.857143,27.77778,rx=0,ry=0,fillColor=Color(0,.501961,0,1),fillOpacity=None,strokeColor=Color(0,.501961,0,1),strokeWidth=1,strokeLineCap=0,strokeLineJoin=0,strokeMiterLimit=0,strokeDashArray=None,strokeOpacity=None))
		self.add(Rect(245,126.3889,60,27.77778,rx=0,ry=0,fillColor=None,fillOpacity=None,strokeColor=Color(0,0,0,1),strokeWidth=0,strokeLineCap=0,strokeLineJoin=0,strokeMiterLimit=0,strokeDashArray=None,strokeOpacity=None))
		self.add(Rect(95,91.66667,2.857143,27.77778,rx=0,ry=0,fillColor=Color(0,0,1,1),fillOpacity=None,strokeColor=Color(0,0,1,1),strokeWidth=1,strokeLineCap=0,strokeLineJoin=0,strokeMiterLimit=0,strokeDashArray=None,strokeOpacity=None))
		self.add(Rect(97.85714,91.66667,2.857143,27.77778,rx=0,ry=0,fillColor=Color(.090909,.090909,1,1),fillOpacity=None,strokeColor=Color(.090909,.090909,1,1),strokeWidth=1,strokeLineCap=0,strokeLineJoin=0,strokeMiterLimit=0,strokeDashArray=None,strokeOpacity=None))
		self.add(Rect(100.7143,91.66667,2.857143,27.77778,rx=0,ry=0,fillColor=Color(.181818,.181818,1,1),fillOpacity=None,strokeColor=Color(.181818,.181818,1,1),strokeWidth=1,strokeLineCap=0,strokeLineJoin=0,strokeMiterLimit=0,strokeDashArray=None,strokeOpacity=None))
		self.add(Rect(103.5714,91.66667,2.857143,27.77778,rx=0,ry=0,fillColor=Color(.272727,.272727,1,1),fillOpacity=None,strokeColor=Color(.272727,.272727,1,1),strokeWidth=1,strokeLineCap=0,strokeLineJoin=0,strokeMiterLimit=0,strokeDashArray=None,strokeOpacity=None))
		self.add(Rect(106.4286,91.66667,2.857143,27.77778,rx=0,ry=0,fillColor=Color(.363636,.363636,1,1),fillOpacity=None,strokeColor=Color(.363636,.363636,1,1),strokeWidth=1,strokeLineCap=0,strokeLineJoin=0,strokeMiterLimit=0,strokeDashArray=None,strokeOpacity=None))
		self.add(Rect(109.2857,91.66667,2.857143,27.77778,rx=0,ry=0,fillColor=Color(.454545,.454545,1,1),fillOpacity=None,strokeColor=Color(.454545,.454545,1,1),strokeWidth=1,strokeLineCap=0,strokeLineJoin=0,strokeMiterLimit=0,strokeDashArray=None,strokeOpacity=None))
		self.add(Rect(112.1429,91.66667,2.857143,27.77778,rx=0,ry=0,fillColor=Color(.545455,.545455,1,1),fillOpacity=None,strokeColor=Color(.545455,.545455,1,1),strokeWidth=1,strokeLineCap=0,strokeLineJoin=0,strokeMiterLimit=0,strokeDashArray=None,strokeOpacity=None))
		self.add(Rect(115,91.66667,2.857143,27.77778,rx=0,ry=0,fillColor=Color(.636364,.636364,1,1),fillOpacity=None,strokeColor=Color(.636364,.636364,1,1),strokeWidth=1,strokeLineCap=0,strokeLineJoin=0,strokeMiterLimit=0,strokeDashArray=None,strokeOpacity=None))
		self.add(Rect(117.8571,91.66667,2.857143,27.77778,rx=0,ry=0,fillColor=Color(.727273,.727273,1,1),fillOpacity=None,strokeColor=Color(.727273,.727273,1,1),strokeWidth=1,strokeLineCap=0,strokeLineJoin=0,strokeMiterLimit=0,strokeDashArray=None,strokeOpacity=None))
		self.add(Rect(120.7143,91.66667,2.857143,27.77778,rx=0,ry=0,fillColor=Color(.818182,.818182,1,1),fillOpacity=None,strokeColor=Color(.818182,.818182,1,1),strokeWidth=1,strokeLineCap=0,strokeLineJoin=0,strokeMiterLimit=0,strokeDashArray=None,strokeOpacity=None))
		self.add(Rect(123.5714,91.66667,2.857143,27.77778,rx=0,ry=0,fillColor=Color(.909091,.909091,1,1),fillOpacity=None,strokeColor=Color(.909091,.909091,1,1),strokeWidth=1,strokeLineCap=0,strokeLineJoin=0,strokeMiterLimit=0,strokeDashArray=None,strokeOpacity=None))
		self.add(Rect(126.4286,91.66667,2.857143,27.77778,rx=0,ry=0,fillColor=Color(1,1,1,1),fillOpacity=None,strokeColor=Color(1,1,1,1),strokeWidth=1,strokeLineCap=0,strokeLineJoin=0,strokeMiterLimit=0,strokeDashArray=None,strokeOpacity=None))
		self.add(Rect(129.2857,91.66667,2.857143,27.77778,rx=0,ry=0,fillColor=Color(.888889,.888889,1,1),fillOpacity=None,strokeColor=Color(.888889,.888889,1,1),strokeWidth=1,strokeLineCap=0,strokeLineJoin=0,strokeMiterLimit=0,strokeDashArray=None,strokeOpacity=None))
		self.add(Rect(132.1429,91.66667,2.857143,27.77778,rx=0,ry=0,fillColor=Color(.777778,.777778,1,1),fillOpacity=None,strokeColor=Color(.777778,.777778,1,1),strokeWidth=1,strokeLineCap=0,strokeLineJoin=0,strokeMiterLimit=0,strokeDashArray=None,strokeOpacity=None))
		self.add(Rect(135,91.66667,2.857143,27.77778,rx=0,ry=0,fillColor=Color(.666667,.666667,1,1),fillOpacity=None,strokeColor=Color(.666667,.666667,1,1),strokeWidth=1,strokeLineCap=0,strokeLineJoin=0,strokeMiterLimit=0,strokeDashArray=None,strokeOpacity=None))
		self.add(Rect(137.8571,91.66667,2.857143,27.77778,rx=0,ry=0,fillColor=Color(.555556,.555556,1,1),fillOpacity=None,strokeColor=Color(.555556,.555556,1,1),strokeWidth=1,strokeLineCap=0,strokeLineJoin=0,strokeMiterLimit=0,strokeDashArray=None,strokeOpacity=None))
		self.add(Rect(140.7143,91.66667,2.857143,27.77778,rx=0,ry=0,fillColor=Color(.444444,.444444,1,1),fillOpacity=None,strokeColor=Color(.444444,.444444,1,1),strokeWidth=1,strokeLineCap=0,strokeLineJoin=0,strokeMiterLimit=0,strokeDashArray=None,strokeOpacity=None))
		self.add(Rect(143.5714,91.66667,2.857143,27.77778,rx=0,ry=0,fillColor=Color(.333333,.333333,1,1),fillOpacity=None,strokeColor=Color(.333333,.333333,1,1),strokeWidth=1,strokeLineCap=0,strokeLineJoin=0,strokeMiterLimit=0,strokeDashArray=None,strokeOpacity=None))
		self.add(Rect(146.4286,91.66667,2.857143,27.77778,rx=0,ry=0,fillColor=Color(.222222,.222222,1,1),fillOpacity=None,strokeColor=Color(.222222,.222222,1,1),strokeWidth=1,strokeLineCap=0,strokeLineJoin=0,strokeMiterLimit=0,strokeDashArray=None,strokeOpacity=None))
		self.add(Rect(149.2857,91.66667,2.857143,27.77778,rx=0,ry=0,fillColor=Color(.111111,.111111,1,1),fillOpacity=None,strokeColor=Color(.111111,.111111,1,1),strokeWidth=1,strokeLineCap=0,strokeLineJoin=0,strokeMiterLimit=0,strokeDashArray=None,strokeOpacity=None))
		self.add(Rect(152.1429,91.66667,2.857143,27.77778,rx=0,ry=0,fillColor=Color(0,0,1,1),fillOpacity=None,strokeColor=Color(0,0,1,1),strokeWidth=1,strokeLineCap=0,strokeLineJoin=0,strokeMiterLimit=0,strokeDashArray=None,strokeOpacity=None))
		self.add(Rect(95,91.66667,60,27.77778,rx=0,ry=0,fillColor=None,fillOpacity=None,strokeColor=Color(0,0,0,1),strokeWidth=0,strokeLineCap=0,strokeLineJoin=0,strokeMiterLimit=0,strokeDashArray=None,strokeOpacity=None))
		self.add(Rect(245,154.1667,2.857143,20.83333,rx=0,ry=0,fillColor=Color(0,0,1,1),fillOpacity=None,strokeColor=Color(0,0,1,1),strokeWidth=1,strokeLineCap=0,strokeLineJoin=0,strokeMiterLimit=0,strokeDashArray=None,strokeOpacity=None))
		self.add(Rect(247.8571,154.1667,2.857143,20.83333,rx=0,ry=0,fillColor=Color(.090909,.090909,1,1),fillOpacity=None,strokeColor=Color(.090909,.090909,1,1),strokeWidth=1,strokeLineCap=0,strokeLineJoin=0,strokeMiterLimit=0,strokeDashArray=None,strokeOpacity=None))
		self.add(Rect(250.7143,154.1667,2.857143,20.83333,rx=0,ry=0,fillColor=Color(.181818,.181818,1,1),fillOpacity=None,strokeColor=Color(.181818,.181818,1,1),strokeWidth=1,strokeLineCap=0,strokeLineJoin=0,strokeMiterLimit=0,strokeDashArray=None,strokeOpacity=None))
		self.add(Rect(253.5714,154.1667,2.857143,20.83333,rx=0,ry=0,fillColor=Color(.272727,.272727,1,1),fillOpacity=None,strokeColor=Color(.272727,.272727,1,1),strokeWidth=1,strokeLineCap=0,strokeLineJoin=0,strokeMiterLimit=0,strokeDashArray=None,strokeOpacity=None))
		self.add(Rect(256.4286,154.1667,2.857143,20.83333,rx=0,ry=0,fillColor=Color(.363636,.363636,1,1),fillOpacity=None,strokeColor=Color(.363636,.363636,1,1),strokeWidth=1,strokeLineCap=0,strokeLineJoin=0,strokeMiterLimit=0,strokeDashArray=None,strokeOpacity=None))
		self.add(Rect(259.2857,154.1667,2.857143,20.83333,rx=0,ry=0,fillColor=Color(.454545,.454545,1,1),fillOpacity=None,strokeColor=Color(.454545,.454545,1,1),strokeWidth=1,strokeLineCap=0,strokeLineJoin=0,strokeMiterLimit=0,strokeDashArray=None,strokeOpacity=None))
		self.add(Rect(262.1429,154.1667,2.857143,20.83333,rx=0,ry=0,fillColor=Color(.545455,.545455,1,1),fillOpacity=None,strokeColor=Color(.545455,.545455,1,1),strokeWidth=1,strokeLineCap=0,strokeLineJoin=0,strokeMiterLimit=0,strokeDashArray=None,strokeOpacity=None))
		self.add(Rect(265,154.1667,2.857143,20.83333,rx=0,ry=0,fillColor=Color(.636364,.636364,1,1),fillOpacity=None,strokeColor=Color(.636364,.636364,1,1),strokeWidth=1,strokeLineCap=0,strokeLineJoin=0,strokeMiterLimit=0,strokeDashArray=None,strokeOpacity=None))
		self.add(Rect(267.8571,154.1667,2.857143,20.83333,rx=0,ry=0,fillColor=Color(.727273,.727273,1,1),fillOpacity=None,strokeColor=Color(.727273,.727273,1,1),strokeWidth=1,strokeLineCap=0,strokeLineJoin=0,strokeMiterLimit=0,strokeDashArray=None,strokeOpacity=None))
		self.add(Rect(270.7143,154.1667,2.857143,20.83333,rx=0,ry=0,fillColor=Color(.818182,.818182,1,1),fillOpacity=None,strokeColor=Color(.818182,.818182,1,1),strokeWidth=1,strokeLineCap=0,strokeLineJoin=0,strokeMiterLimit=0,strokeDashArray=None,strokeOpacity=None))
		self.add(Rect(273.5714,154.1667,2.857143,20.83333,rx=0,ry=0,fillColor=Color(.909091,.909091,1,1),fillOpacity=None,strokeColor=Color(.909091,.909091,1,1),strokeWidth=1,strokeLineCap=0,strokeLineJoin=0,strokeMiterLimit=0,strokeDashArray=None,strokeOpacity=None))
		self.add(Rect(276.4286,154.1667,2.857143,20.83333,rx=0,ry=0,fillColor=Color(1,1,1,1),fillOpacity=None,strokeColor=Color(1,1,1,1),strokeWidth=1,strokeLineCap=0,strokeLineJoin=0,strokeMiterLimit=0,strokeDashArray=None,strokeOpacity=None))
		self.add(Rect(279.2857,154.1667,2.857143,20.83333,rx=0,ry=0,fillColor=Color(.888889,.888889,1,1),fillOpacity=None,strokeColor=Color(.888889,.888889,1,1),strokeWidth=1,strokeLineCap=0,strokeLineJoin=0,strokeMiterLimit=0,strokeDashArray=None,strokeOpacity=None))
		self.add(Rect(282.1429,154.1667,2.857143,20.83333,rx=0,ry=0,fillColor=Color(.777778,.777778,1,1),fillOpacity=None,strokeColor=Color(.777778,.777778,1,1),strokeWidth=1,strokeLineCap=0,strokeLineJoin=0,strokeMiterLimit=0,strokeDashArray=None,strokeOpacity=None))
		self.add(Rect(285,154.1667,2.857143,20.83333,rx=0,ry=0,fillColor=Color(.666667,.666667,1,1),fillOpacity=None,strokeColor=Color(.666667,.666667,1,1),strokeWidth=1,strokeLineCap=0,strokeLineJoin=0,strokeMiterLimit=0,strokeDashArray=None,strokeOpacity=None))
		self.add(Rect(287.8571,154.1667,2.857143,20.83333,rx=0,ry=0,fillColor=Color(.555556,.555556,1,1),fillOpacity=None,strokeColor=Color(.555556,.555556,1,1),strokeWidth=1,strokeLineCap=0,strokeLineJoin=0,strokeMiterLimit=0,strokeDashArray=None,strokeOpacity=None))
		self.add(Rect(290.7143,154.1667,2.857143,20.83333,rx=0,ry=0,fillColor=Color(.444444,.444444,1,1),fillOpacity=None,strokeColor=Color(.444444,.444444,1,1),strokeWidth=1,strokeLineCap=0,strokeLineJoin=0,strokeMiterLimit=0,strokeDashArray=None,strokeOpacity=None))
		self.add(Rect(293.5714,154.1667,2.857143,20.83333,rx=0,ry=0,fillColor=Color(.333333,.333333,1,1),fillOpacity=None,strokeColor=Color(.333333,.333333,1,1),strokeWidth=1,strokeLineCap=0,strokeLineJoin=0,strokeMiterLimit=0,strokeDashArray=None,strokeOpacity=None))
		self.add(Rect(296.4286,154.1667,2.857143,20.83333,rx=0,ry=0,fillColor=Color(.222222,.222222,1,1),fillOpacity=None,strokeColor=Color(.222222,.222222,1,1),strokeWidth=1,strokeLineCap=0,strokeLineJoin=0,strokeMiterLimit=0,strokeDashArray=None,strokeOpacity=None))
		self.add(Rect(299.2857,154.1667,2.857143,20.83333,rx=0,ry=0,fillColor=Color(.111111,.111111,1,1),fillOpacity=None,strokeColor=Color(.111111,.111111,1,1),strokeWidth=1,strokeLineCap=0,strokeLineJoin=0,strokeMiterLimit=0,strokeDashArray=None,strokeOpacity=None))
		self.add(Rect(302.1429,154.1667,2.857143,20.83333,rx=0,ry=0,fillColor=Color(0,0,1,1),fillOpacity=None,strokeColor=Color(0,0,1,1),strokeWidth=1,strokeLineCap=0,strokeLineJoin=0,strokeMiterLimit=0,strokeDashArray=None,strokeOpacity=None))
		self.add(Rect(245,154.1667,60,20.83333,rx=0,ry=0,fillColor=None,fillOpacity=None,strokeColor=Color(0,0,0,1),strokeWidth=0,strokeLineCap=0,strokeLineJoin=0,strokeMiterLimit=0,strokeDashArray=None,strokeOpacity=None))
		self.add(Line(50,50,350,50,strokeColor=Color(0,0,0,1),strokeWidth=1,strokeLineCap=0,strokeLineJoin=0,strokeMiterLimit=0,strokeDashArray=None,strokeOpacity=None))
		self.add(Line(50,50,50,45,strokeColor=Color(0,0,0,1),strokeWidth=1,strokeLineCap=0,strokeLineJoin=0,strokeMiterLimit=10,strokeDashArray=None,strokeOpacity=None))
		self.add(Line(200,50,200,45,strokeColor=Color(0,0,0,1),strokeWidth=1,strokeLineCap=0,strokeLineJoin=0,strokeMiterLimit=10,strokeDashArray=None,strokeOpacity=None))
		self.add(Line(350,50,350,45,strokeColor=Color(0,0,0,1),strokeWidth=1,strokeLineCap=0,strokeLineJoin=0,strokeMiterLimit=10,strokeDashArray=None,strokeOpacity=None))
		v0=self._nn(Group())
		v0.transform = (1,0,0,1,125,45)
		v0.add(String(-20,-4,'Ying',textAnchor='start',fontName='Times-Roman',fontSize=10,fillColor=Color(0,0,0,1)))
		v0=self._nn(Group())
		v0.transform = (1,0,0,1,275,45)
		v0.add(String(-21.66,-4,'Yang',textAnchor='start',fontName='Times-Roman',fontSize=10,fillColor=Color(0,0,0,1)))
		self.add(Line(50,50,50,175,strokeColor=Color(0,0,0,1),strokeWidth=1,strokeLineCap=0,strokeLineJoin=0,strokeMiterLimit=0,strokeDashArray=None,strokeOpacity=None))
		self.add(Line(50,50,45,50,strokeColor=Color(0,0,0,1),strokeWidth=1,strokeLineCap=0,strokeLineJoin=0,strokeMiterLimit=10,strokeDashArray=None,strokeOpacity=None))
		self.add(Line(50,84.72222,45,84.72222,strokeColor=Color(0,0,0,1),strokeWidth=1,strokeLineCap=0,strokeLineJoin=0,strokeMiterLimit=10,strokeDashArray=None,strokeOpacity=None))
		self.add(Line(50,119.4444,45,119.4444,strokeColor=Color(0,0,0,1),strokeWidth=1,strokeLineCap=0,strokeLineJoin=0,strokeMiterLimit=10,strokeDashArray=None,strokeOpacity=None))
		self.add(Line(50,154.1667,45,154.1667,strokeColor=Color(0,0,0,1),strokeWidth=1,strokeLineCap=0,strokeLineJoin=0,strokeMiterLimit=10,strokeDashArray=None,strokeOpacity=None))
		v0=self._nn(Group())
		v0.transform = (1,0,0,1,45,50)
		v0.add(String(-5,-4,'0',textAnchor='start',fontName='Times-Roman',fontSize=10,fillColor=Color(0,0,0,1)))
		v0=self._nn(Group())
		v0.transform = (1,0,0,1,45,84.72222)
		v0.add(String(-10,-4,'50',textAnchor='start',fontName='Times-Roman',fontSize=10,fillColor=Color(0,0,0,1)))
		v0=self._nn(Group())
		v0.transform = (1,0,0,1,45,119.4444)
		v0.add(String(-15,-4,'100',textAnchor='start',fontName='Times-Roman',fontSize=10,fillColor=Color(0,0,0,1)))
		v0=self._nn(Group())
		v0.transform = (1,0,0,1,45,154.1667)
		v0.add(String(-15,-4,'150',textAnchor='start',fontName='Times-Roman',fontSize=10,fillColor=Color(0,0,0,1)))


if __name__=="__main__": #NORUNTESTS
	ExplodedDrawing_Drawing().save(formats=['pdf'],outDir='.',fnRoot=None)



#!/usr/bin/env python
#Copyright ReportLab Europe Ltd. 2000-2017
#see license.txt for license details

"""This is a test on a package level that find all modules,
classes, methods and functions that do not have a doc string
and lists them in individual log files.

Currently, methods with leading and trailing double underscores
are skipped.
"""
from reportlab.lib.testutils import setOutDir,SecureTestCase, GlobDirectoryWalker, outputfile, printLocation
setOutDir(__name__)
import os, sys, glob, re, unittest, inspect
import reportlab
from reportlab.lib.utils import rl_exec, isPyPy

def typ2is(typ):
    return getattr(inspect,'is'+typ)

_typ2key={
        'module':lambda x: (x[0],getattr(x[1],'__name__',''),getattr(x[1],'__path__',getattr(x,'__file__',''))),
        'class':lambda x: (x[0],getattr(x[1],'__name__',''),getattr(x[1],'__module__','')),
        'method':lambda x: (x[0],getattr(x[1],'__name__',''),getattr(x[1],'__module__','')),
        'function':lambda x: (x[0],getattr(x[1],'__name__',''),'???' if isPyPy else x[1].__code__.co_filename),
        }
def typ2key(typ):
    return _typ2key[typ]

def obj2typ(obj):
    for typ in ('function','module','class','method'):
        if typ2is(typ)(obj): return typ
    return None


def getClass(obj):
    try:
        return obj.__self__.__class__
    except:
        try:
            return obj.im_class
        except:
            return None

from pkgutil import iter_modules
def walk_packages_ex(path=None, prefix='', onerror=None, cond=None):
    def seen(p, m={}):
        if p in m:
            return True
        m[p] = True

    for importer, name, ispkg in iter_modules(path, prefix):
        if cond and not cond(importer,name,ispkg): continue
        yield importer, name, ispkg

        if ispkg:
            try:
                __import__(name)
            except ImportError:
                if onerror is not None:
                    onerror(name)
            except Exception:
                if onerror is not None:
                    onerror(name)
                else:
                    raise
            else:
                path = getattr(sys.modules[name], '__path__', None) or []

                # don't traverse path items we've seen before
                path = [p for p in path if not seen(p)]

                for item in walk_packages_ex(path, name+'.', onerror, cond):
                    yield item

def rl_module(i,name,pkg):
    return name=='reportlab' or name.startswith('reportlab.')

rl_modules = None
def getRLModules():
    "Get a list of all objects defined *somewhere* in a package."
    global rl_modules
    if rl_modules is None:
        rl_modules = []
        for _,name,_ in walk_packages_ex(cond=rl_module):
            rl_modules.append(name)
    return rl_modules

def getObjects(objects,lookup,mName,modBn,tobj):
    ttyp = obj2typ(tobj)
    for n in dir(tobj):
        obj = getattr(tobj,n,None)
        try:
            if obj in lookup: continue
        except:
            continue
        typ = obj2typ(obj)
        if typ in ('function','method'):
            try:
                cond = not isPyPy and os.path.splitext(obj.__code__.co_filename)[0]==modBn
            except:
                pass
            else:
                if cond:
                    lookup[obj] = 1
                    objects.setdefault(typ if typ=='function' and ttyp=='module' else 'method',[]).append((mName,obj))
        elif typ=='class':
            if obj.__module__==mName:
                lookup[obj] = 1
                objects.setdefault(typ,[]).append((mName,obj))
                getObjects(objects,lookup,mName,modBn,obj)

def getModuleObjects(modules):
    objects = {}
    lookup = {}
    for mName in modules:
        try:
            NS = {}
            rl_exec("import %s as module" % mName,NS)
        except ImportError:
            continue
        else:
            module = NS['module']
        if module in lookup: continue

        lookup[module] = 1
        objects.setdefault('module',[]).append((mName, module))
        modBn = os.path.splitext(module.__file__)[0]
        getObjects(objects,lookup,mName,modBn,module)
    return objects

class DocstringTestCase(SecureTestCase):
    "Testing if objects in the ReportLab package have docstrings."

    def setUp(self):
        SecureTestCase.setUp(self)
        self.modules = getRLModules()
        self.objects = getModuleObjects(self.modules)

    def _writeLogFile(self, typ):
        "Write log file for different kind of documentable objects."

        objects = self.objects.get(typ,[])
        objects.sort(key=typ2key(typ))

        expl = {'function':'functions',
                'class':'classes',
                'method':'methods',
                'module':'modules'}[typ]

        path = outputfile("test_docstrings-%s.log" % expl)
        file = open(path, 'w')
        file.write('No doc strings found for the following %s below.\n\n' % expl)
        p = re.compile('__.+__')

        lines = []
        for name, obj in objects:
            if typ == 'method':
                n = obj.__name__
                # Skip names with leading and trailing double underscores.
                if p.match(n):
                    continue

            if not obj.__doc__ or len(obj.__doc__) == 0:
                if typ == 'function':
                    lines.append("%s.%s\n" % (name, obj.__name__))
                elif typ == 'class':
                    lines.append("%s.%s\n" % (obj.__module__, getattr(obj,'__qualname__',getattr(obj,'__name__','[unknown __name__]'))))
                else:
                    lines.append("%s\n" % (getattr(obj,'__qualname__',getattr(obj,'__name__','[unknown __name__]'))))

        lines.sort()
        for line in lines:
            file.write(line)

        file.close()

    def test0(self):
        "Test if functions have a doc string."
        self._writeLogFile('function')

    def test1(self):
        "Test if classes have a doc string."
        self._writeLogFile('class')

    def test2(self):
        "Test if methods have a doc string."
        self._writeLogFile('method')

    def test3(self):
        "Test if modules have a doc string."
        self._writeLogFile('module')

def makeSuite():
    suite = unittest.TestSuite()
    loader = unittest.TestLoader()
    if sys.platform[:4] != 'java': suite.addTest(loader.loadTestsFromTestCase(DocstringTestCase))
    return suite

#noruntests
if __name__ == "__main__":
    unittest.TextTestRunner().run(makeSuite())
    printLocation()



#Autogenerated by ReportLab guiedit do not edit
from reportlab.graphics.shapes import _DrawingEditorMixin, Drawing, Group, String
from reportlab.lib.colors import Color, CMYKColor, PCMYKColor

class ExplodedDrawing_Drawing(_DrawingEditorMixin,Drawing):
	def __init__(self,width=400,height=200,*args,**kw):
		Drawing.__init__(self,width,height,*args,**kw)
		self.transform = (1,0,0,1,0,0)
		self.add(String(34,34,'Hello World',textAnchor='start',fontName='Times-Roman',fontSize=12,fillColor=Color(0,0,0,1)))
		self.add(String(42,42,'Hello World',textAnchor='start',fontName='Times-Roman',fontSize=16,fillColor=Color(0,0,0,1)))
		self.add(String(50,50,'Hello World',textAnchor='start',fontName='Times-Roman',fontSize=20,fillColor=Color(0,0,0,1)))
		self.add(String(58,58,'Hello World',textAnchor='start',fontName='Times-Roman',fontSize=24,fillColor=Color(0,0,0,1)))
		self.add(String(66,66,'Hello World',textAnchor='start',fontName='Times-Roman',fontSize=28,fillColor=Color(0,0,0,1)))
		self.add(String(74,74,'Hello World',textAnchor='start',fontName='Times-Roman',fontSize=32,fillColor=Color(0,0,0,1)))
		self.add(String(150,150,'Hello World',textAnchor='start',fontName='Vera',fontSize=36,fillColor=Color(0,0,0,1)))


if __name__=="__main__": #NORUNTESTS
	ExplodedDrawing_Drawing().save(formats=['pdf'],outDir='.',fnRoot=None)



#Autogenerated by ReportLab guiedit do not edit
from reportlab.graphics.shapes import _DrawingEditorMixin, Drawing, Group, Wedge, String, Rect
from reportlab.lib.colors import Color, CMYKColor, PCMYKColor

class ExplodedDrawing_Drawing(_DrawingEditorMixin,Drawing):
	def __init__(self,width=300,height=150,*args,**kw):
		Drawing.__init__(self,width,height,*args,**kw)
		self.transform = (1,0,0,1,0,0)
		self.add(Wedge(200,100,50,89.55056,90,yradius=50,annular=False,fillColor=Color(0,.545098,.545098,1),fillOpacity=None,strokeColor=Color(0,0,0,1),strokeWidth=.5,strokeLineCap=0,strokeLineJoin=1,strokeMiterLimit=0,strokeDashArray=None,strokeOpacity=None))
		self.add(Wedge(200,100,50,67.07865,89.55056,yradius=50,annular=False,fillColor=Color(.541176,.168627,.886275,1),fillOpacity=None,strokeColor=Color(0,0,0,1),strokeWidth=.5,strokeLineCap=0,strokeLineJoin=1,strokeMiterLimit=0,strokeDashArray=None,strokeOpacity=None))
		self.add(Wedge(200,100,50,22.13483,67.07865,yradius=50,annular=False,fillColor=Color(0,0,1,1),fillOpacity=None,strokeColor=Color(0,0,0,1),strokeWidth=.5,strokeLineCap=0,strokeLineJoin=1,strokeMiterLimit=0,strokeDashArray=None,strokeOpacity=None))
		self.add(Wedge(219.9997,99.88234,50,-22.80899,22.13483,yradius=50,annular=False,fillColor=Color(0,1,1,1),fillOpacity=None,strokeColor=Color(0,0,0,1),strokeWidth=2,strokeLineCap=0,strokeLineJoin=1,strokeMiterLimit=0,strokeDashArray=[2,2],strokeOpacity=None))
		self.add(Wedge(200,100,50,-67.75281,-22.80899,yradius=50,annular=False,fillColor=Color(1,.752941,.796078,1),fillOpacity=None,strokeColor=Color(0,0,0,1),strokeWidth=.5,strokeLineCap=0,strokeLineJoin=1,strokeMiterLimit=0,strokeDashArray=None,strokeOpacity=None))
		self.add(Wedge(200,100,50,-112.6966,-67.75281,yradius=50,annular=False,fillColor=Color(1,0,1,1),fillOpacity=None,strokeColor=Color(0,0,0,1),strokeWidth=.5,strokeLineCap=0,strokeLineJoin=1,strokeMiterLimit=0,strokeDashArray=None,strokeOpacity=None))
		self.add(Wedge(200,100,50,-157.6404,-112.6966,yradius=50,annular=False,fillColor=Color(1,1,0,1),fillOpacity=None,strokeColor=Color(0,0,0,1),strokeWidth=.5,strokeLineCap=0,strokeLineJoin=1,strokeMiterLimit=0,strokeDashArray=None,strokeOpacity=None))
		self.add(Wedge(200,100,50,-202.5843,-157.6404,yradius=50,annular=False,fillColor=Color(0,.545098,.545098,1),fillOpacity=None,strokeColor=Color(0,0,0,1),strokeWidth=.5,strokeLineCap=0,strokeLineJoin=1,strokeMiterLimit=0,strokeDashArray=None,strokeOpacity=None))
		self.add(Wedge(200,100,50,-247.5281,-202.5843,yradius=50,annular=False,fillColor=Color(.541176,.168627,.886275,1),fillOpacity=None,strokeColor=Color(0,0,0,1),strokeWidth=.5,strokeLineCap=0,strokeLineJoin=1,strokeMiterLimit=0,strokeDashArray=None,strokeOpacity=None))
		self.add(Wedge(200,100,50,-270,-247.5281,yradius=50,annular=False,fillColor=Color(0,0,1,1),fillOpacity=None,strokeColor=Color(0,0,0,1),strokeWidth=.5,strokeLineCap=0,strokeLineJoin=1,strokeMiterLimit=0,strokeDashArray=None,strokeOpacity=None))
		self.add(String(200.2353,159.9995,'0',textAnchor='middle',fontName='Times-Roman',fontSize=10,fillColor=Color(0,0,0,1)))
		self.add(String(212.1523,158.7565,'ä',textAnchor='middle',fontName='Helvetica',fontSize=10,fillColor=Color(0,0,0,1)))
		self.add(String(242.7166,142.1342,'b',textAnchor='middle',fontName='Times-Roman',fontSize=10,fillColor=Color(0,0,0,1)))
		self.add(String(307.4981,99.36757,'c',textAnchor='middle',fontName='Times-Roman',fontSize=10,fillColor=Color(1,0,0,1)))
		self.add(String(242.2179,57.3661,'d',textAnchor='middle',fontName='Times-Roman',fontSize=10,fillColor=Color(0,0,0,1)))
		self.add(String(199.7647,40.00046,'e',textAnchor='middle',fontName='Times-Roman',fontSize=10,fillColor=Color(0,0,0,1)))
		self.add(String(157.449,57.69858,'f',textAnchor='middle',fontName='Times-Roman',fontSize=10,fillColor=Color(0,0,0,1)))
		self.add(String(140.0001,100.1177,'g',textAnchor='middle',fontName='Times-Roman',fontSize=10,fillColor=Color(0,0,0,1)))
		self.add(String(157.6152,142.468,'h',textAnchor='middle',fontName='Helvetica',fontSize=10,fillColor=Color(0,0,0,1)))
		self.add(String(188.309,158.85,'i',textAnchor='middle',fontName='Times-Roman',fontSize=10,fillColor=Color(0,0,0,1)))
		self.add(Rect(310,140,20,5,rx=0,ry=0,fillColor=Color(0,.545098,.545098,1),fillOpacity=None,strokeColor=Color(0,0,0,1),strokeWidth=.5,strokeLineCap=0,strokeLineJoin=0,strokeMiterLimit=0,strokeDashArray=None,strokeOpacity=None))
		self.add(String(295,139.085,'0',textAnchor='start',fontName='Times-Roman',fontSize=10,fillColor=Color(0,0,0,1)))
		self.add(Rect(310,120,20,5,rx=0,ry=0,fillColor=Color(.541176,.168627,.886275,1),fillOpacity=None,strokeColor=Color(0,0,0,1),strokeWidth=.5,strokeLineCap=0,strokeLineJoin=0,strokeMiterLimit=0,strokeDashArray=None,strokeOpacity=None))
		self.add(String(295,119.085,'ä',textAnchor='start',fontName='Times-Roman',fontSize=10,fillColor=Color(0,0,0,1)))
		self.add(Rect(310,100,20,5,rx=0,ry=0,fillColor=Color(0,0,1,1),fillOpacity=None,strokeColor=Color(0,0,0,1),strokeWidth=.5,strokeLineCap=0,strokeLineJoin=0,strokeMiterLimit=0,strokeDashArray=None,strokeOpacity=None))
		self.add(String(295,99.085,'b',textAnchor='start',fontName='Times-Roman',fontSize=10,fillColor=Color(0,0,0,1)))
		self.add(Rect(350,140,20,5,rx=0,ry=0,fillColor=Color(0,1,1,1),fillOpacity=None,strokeColor=Color(0,0,0,1),strokeWidth=2,strokeLineCap=0,strokeLineJoin=0,strokeMiterLimit=0,strokeDashArray=[2,2],strokeOpacity=None))
		self.add(String(335,139.085,'c',textAnchor='start',fontName='Times-Roman',fontSize=10,fillColor=Color(0,0,0,1)))
		self.add(Rect(350,120,20,5,rx=0,ry=0,fillColor=Color(1,.752941,.796078,1),fillOpacity=None,strokeColor=Color(0,0,0,1),strokeWidth=.5,strokeLineCap=0,strokeLineJoin=0,strokeMiterLimit=0,strokeDashArray=None,strokeOpacity=None))
		self.add(String(335,119.085,'d',textAnchor='start',fontName='Times-Roman',fontSize=10,fillColor=Color(0,0,0,1)))
		self.add(Rect(350,100,20,5,rx=0,ry=0,fillColor=Color(1,0,1,1),fillOpacity=None,strokeColor=Color(0,0,0,1),strokeWidth=.5,strokeLineCap=0,strokeLineJoin=0,strokeMiterLimit=0,strokeDashArray=None,strokeOpacity=None))
		self.add(String(335,99.085,'e',textAnchor='start',fontName='Times-Roman',fontSize=10,fillColor=Color(0,0,0,1)))
		self.add(Rect(390,140,20,5,rx=0,ry=0,fillColor=Color(1,1,0,1),fillOpacity=None,strokeColor=Color(0,0,0,1),strokeWidth=.5,strokeLineCap=0,strokeLineJoin=0,strokeMiterLimit=0,strokeDashArray=None,strokeOpacity=None))
		self.add(String(375,139.085,'f',textAnchor='start',fontName='Times-Roman',fontSize=10,fillColor=Color(0,0,0,1)))
		self.add(Rect(390,120,20,5,rx=0,ry=0,fillColor=Color(0,.545098,.545098,1),fillOpacity=None,strokeColor=Color(0,0,0,1),strokeWidth=.5,strokeLineCap=0,strokeLineJoin=0,strokeMiterLimit=0,strokeDashArray=None,strokeOpacity=None))
		self.add(String(375,119.085,'g',textAnchor='start',fontName='Times-Roman',fontSize=10,fillColor=Color(0,0,0,1)))
		self.add(Rect(390,100,20,5,rx=0,ry=0,fillColor=Color(.541176,.168627,.886275,1),fillOpacity=None,strokeColor=Color(0,0,0,1),strokeWidth=.5,strokeLineCap=0,strokeLineJoin=0,strokeMiterLimit=0,strokeDashArray=None,strokeOpacity=None))
		self.add(String(375,99.085,'h',textAnchor='start',fontName='Times-Roman',fontSize=10,fillColor=Color(0,0,0,1)))
		self.add(Rect(430,140,20,5,rx=0,ry=0,fillColor=Color(0,0,1,1),fillOpacity=None,strokeColor=Color(0,0,0,1),strokeWidth=.5,strokeLineCap=0,strokeLineJoin=0,strokeMiterLimit=0,strokeDashArray=None,strokeOpacity=None))
		self.add(String(415,139.085,'i',textAnchor='start',fontName='Times-Roman',fontSize=10,fillColor=Color(0,0,0,1)))


if __name__=="__main__": #NORUNTESTS
	ExplodedDrawing_Drawing().save(formats=['pdf'],outDir='.',fnRoot=None)



"""Tests (incomplete) for the reportlab.lib.validators module.
"""
from reportlab.lib.testutils import setOutDir,makeSuiteForClasses, printLocation
setOutDir(__name__)
import unittest
from reportlab.lib import colors
from reportlab.lib import validators

class ValidatorTestCase(unittest.TestCase):
    "Test validating functions."

    def test0(self):
        "Test isBoolean validator."

        msg = "Validation failed for 'boolean' %s!"

        booleans = [0, 1, 'yes','no','true','false']
        badbooleans = ['a',3,-1,()]
        isBoolean = validators.isBoolean
        for b in booleans:
            assert isBoolean(b) == 1, msg % str(b)
        for b in badbooleans:
            assert isBoolean(b) == 0, msg % str(b)


    def test1(self):
        "Test isNumber validator."

        msg = 'Validation failed for number %s!'

        numbers = [0, 1, 2, -1, -2, 0.0, 0.1, -0.1]
        badNumbers = ['aaa',(1,1),(1+1j),colors]
        isNumber = validators.isNumber
        isListOfNumbers = validators.isListOfNumbers
        for n in numbers:
            assert isNumber(n) == 1, msg % str(n)
        for n in badNumbers:
            assert isNumber(n) == 0, msg % str(n)

        msg = 'Validation failed for numbers %s!'

        assert isListOfNumbers(numbers) == 1, msg % str(numbers)
        assert isListOfNumbers(badNumbers) == 0, msg % str(badNumbers)
        assert isListOfNumbers(numbers+[colors]) == 0, msg % str(numbers+[colors])


    def test2(self):
        "Test isNumberOrNone validator."

        msg = 'Validation failed for number %s!'

        numbers = [None, 0, 1, 2, -1, -2, 0.0, 0.1, -0.1] #, 2L, -2L]
        isNumberOrNone = validators.isNumberOrNone
        for n in numbers:
            assert isNumberOrNone(n) == 1, msg % str(n)


    def test4(self):
        "Test isString validator."

        msg = 'Validation failed for string %s!'

        strings = ['', '\n', '  ', 'foo', '""']
        badStrings = [1,2.0,None,('a','b')]
        isString = validators.isString
        isListOfStrings = validators.isListOfStrings
        for s in strings:
            assert isString(s) == 1, msg % str(s)
        for s in badStrings:
            assert isString(s) == 0, msg % str(s)

        msg = 'Validation failed for strings %s!'

        assert isListOfStrings(strings) == 1, msg % str(strings)
        assert isListOfStrings(badStrings) == 0, msg % str(badStrings)
        assert isListOfStrings(strings+[1]) == 0, msg % str(strings+[1])


    def test5(self):
        "Test isTextAnchor validator."

        msg = 'Validation failed for text anchor %s!'

        strings = ['start', 'middle', 'end']
        isTextAnchor = validators.isTextAnchor
        for s in strings:
            assert isTextAnchor(s) == 1, msg % s

        """
        def isListOfNumbersOrNone(x):
        def isListOfShapes(x):
        def isListOfStrings(x):
        def isListOfStringsOrNone(x):
        def isTransform(x):
        def isColor(x):
        def isColorOrNone(x):
        def isValidChild(x):
        class OneOf:
        class SequenceOf:
        """


    def test6(self):
        "Test OneOf validator."

        msg = 'Validation failed for OneOf %s!'

        choices = ('clockwise', 'anticlockwise')
        OneOf = validators.OneOf(choices)
        for c in choices:
            assert OneOf(c) == 1, msg % c
        for c in ('a', 'b', 'c'):
            assert OneOf(c) == 0, msg % c

        OneOf = validators.OneOf('clockwise', 'anticlockwise')
        for c in choices:
            assert OneOf(c) == 1, msg % c
        for c in ('a', 'b', 'c'):
            assert OneOf(c) == 0, msg % c

        try:
            validators.OneOf(choices,'bongo')
            raise AssertionError("OneOf failed to detect bad arguments")
        except ValueError:
            pass


    def test7(self):
        "Test isInt validator"

        msg = 'Validation failed for isInt %s!'

        isInt = validators.isInt
        for c in (1,2,-3,0,'-4','4'):
            assert isInt(c), msg % str(c)

        for c in (1.2,0.0,-3.0,'-4.0','4.4','AAAA'):
            assert not isInt(c), msg % str(c)


    def test8(self):
        "test Sequence of validator"

        msg = 'Validation failed for SequenceOf %s!'

        v=validators.SequenceOf(validators.OneOf(('eps','pdf','png','gif','jpg','tif')),lo=1,hi=3,emptyOK=0)
        for c in (['png'],('eps',),('eps','pdf')):
            assert v(c), msg % str(c)
        v._lo = 2
        for c in ([],(),('eps'),('eps','pdf','a'),['eps','pdf','png','gif']):
            assert not v(c), msg % str(c)
        v._emptyOK=1
        for c in ([],(),('eps','pdf')):
            assert v(c), msg % str(c)


def makeSuite():
    return makeSuiteForClasses(ValidatorTestCase)


#noruntests
if __name__ == "__main__":
    unittest.TextTestRunner().run(makeSuite())
    printLocation()



#Copyright ReportLab Europe Ltd. 2000-2017
#see license.txt for license details
#history https://hg.reportlab.com/hg-public/reportlab/log/tip/tools/docco/t_parse.py
"""
Template parsing module inspired by REXX (with thanks to Donn Cave for discussion).

Template initialization has the form:
   T = Template(template_string, wild_card_marker, single_char_marker,
             x = regex_x, y = regex_y, ...)
Parsing has the form
   ([match1, match2, ..., matchn], lastindex) = T.PARSE(string)

Only the first argument is mandatory.

The resultant object efficiently parses strings that match the template_string,
giving a list of substrings that correspond to each "directive" of the template.

Template directives:

  Wildcard:
    The template may be initialized with a wildcard that matches any string
    up to the string matching the next directive (which may not be a wild
    card or single character marker) or the next literal sequence of characters
    of the template.  The character that represents a wildcard is specified
    by the wild_card_marker parameter, which has no default.

    For example, using X as the wildcard:


    >>> T = Template("prefixXinteriorX", "X")
    >>> T.PARSE("prefix this is before interior and this is after")
    ([' this is before ', ' and this is after'], 47)
    >>> T = Template("<X>X<X>", "X")
    >>> T.PARSE('<A HREF="index.html">go to index</A>')
    (['A HREF="index.html"', 'go to index', '/A'], 36)

    Obviously the character used to represent the wildcard must be distinct
    from the characters used to represent literals or other directives.

  Fixed length character sequences:
    The template may have a marker character which indicates a fixed
    length field.  All adjacent instances of this marker will be matched
    by a substring of the same length in the parsed string.  For example:

      >>> T = Template("NNN-NN-NNNN", single_char_marker="N")
      >>> T.PARSE("1-2-34-5-12")
      (['1-2', '34', '5-12'], 11)
      >>> T.PARSE("111-22-3333")
      (['111', '22', '3333'], 11)
      >>> T.PARSE("1111-22-3333")
      ValueError: literal not found at (3, '-')

    A template may have multiple fixed length markers, which allows fixed
    length fields to be adjacent, but recognized separately.  For example:

      >>> T = Template("MMDDYYX", "X", "MDY")
      >>> T.PARSE("112489 Somebody's birthday!")
      (['11', '24', '89', " Somebody's birthday!"], 27)

  Regular expression markers:
    The template may have markers associated with regular expressions.
    the regular expressions may be either string represenations of compiled.
    For example:
      >>> T = Template("v: s i", v=id, s=str, i=int)
      >>> T.PARSE("this_is_an_identifier: 'a string' 12344")
      (['this_is_an_identifier', "'a string'", '12344'], 39)
      >>>

    Here id, str, and int are regular expression conveniences provided by
    this module.

  Directive markers may be mixed and matched, except that wildcards cannot precede
  wildcards or single character markers.
  Example:
>>> T = Template("ssnum: NNN-NN-NNNN, fn=X, ln=X, age=I, quote=Q", "X", "N", I=int, Q=str)
>>> T.PARSE("ssnum: 123-45-6789, fn=Aaron, ln=Watters, age=13, quote='do be do be do'")
(['123', '45', '6789', 'Aaron', 'Watters', '13', "'do be do be do'"], 72)
>>>

"""

import re, string
from reportlab.lib.utils import ascii_letters

#
# template parsing
#
# EG: T = Template("(NNN)NNN-NNNN X X", "X", "N")
#     ([area, exch, ext, fn, ln], index) = T.PARSE("(908)949-2726 Aaron Watters")
#
class Template:

   def __init__(self,
                template,
                wild_card_marker=None,
                single_char_marker=None,
                **marker_to_regex_dict):
       self.template = template
       self.wild_card = wild_card_marker
       self.char = single_char_marker
       # determine the set of markers for this template
       markers = list(marker_to_regex_dict.keys())
       if wild_card_marker:
          markers.append(wild_card_marker)
       if single_char_marker:
          for ch in single_char_marker: # allow multiple scm's
              markers.append(ch)
          self.char = single_char_primary = single_char_marker[0]
       self.markers = markers
       for mark in markers:
           if len(mark)>1:
              raise ValueError("Marks must be single characters: "+repr(mark))
       # compile the regular expressions if needed
       self.marker_dict = marker_dict = {}
       for mark, rgex in marker_to_regex_dict.items():
           if isinstance(rgex,str):
              rgex = re.compile(rgex)
           marker_dict[mark] = rgex
       # determine the parse sequence
       parse_seq = []
       # dummy last char
       lastchar = None
       index = 0
       last = len(template)
       # count the number of directives encountered
       ndirectives = 0
       while index<last:
          start = index
          thischar = template[index]
          # is it a wildcard?
          if thischar == wild_card_marker:
             if lastchar == wild_card_marker:
                raise ValueError("two wild cards in sequence is not allowed")
             parse_seq.append( (wild_card_marker, None) )
             index = index+1
             ndirectives = ndirectives+1
          # is it a sequence of single character markers?
          elif single_char_marker and thischar in single_char_marker:
             if lastchar == wild_card_marker:
                raise ValueError("wild card cannot precede single char marker")
             while index<last and template[index] == thischar:
                index = index+1
             parse_seq.append( (single_char_primary, index-start) )
             ndirectives = ndirectives+1
          # is it a literal sequence?
          elif not thischar in markers:
             while index<last and not template[index] in markers:
                index = index+1
             parse_seq.append( (None, template[start:index]) )
          # otherwise it must be a re marker
          else:
             rgex = marker_dict[thischar]
             parse_seq.append( (thischar, rgex) )
             ndirectives = ndirectives+1
             index = index+1
          lastchar = template[index-1]
       self.parse_seq = parse_seq
       self.ndirectives = ndirectives

   def PARSE(self, s, start=0):
       ndirectives = self.ndirectives
       wild_card = self.wild_card
       single_char = self.char
       parse_seq = self.parse_seq
       lparse_seq = len(parse_seq) - 1
       # make a list long enough for substitutions for directives
       result = [None] * ndirectives
       current_directive_index = 0
       currentindex = start
       # scan through the parse sequence, recognizing
       for parse_index in range(lparse_seq + 1):
           (indicator, data) = parse_seq[parse_index]
           # is it a literal indicator?
           if indicator is None:
              if s.find(data, currentindex) != currentindex:
                 raise ValueError("literal not found at "+repr((currentindex,data)))
              currentindex = currentindex + len(data)
           else:
              # anything else is a directive
              # is it a wildcard?
              if indicator == wild_card:
                 # if it is the last directive then it matches the rest of the string
                 if parse_index == lparse_seq:
                    last = len(s)
                 # otherwise must look at next directive to find end of wildcard
                 else:
                    # next directive must be re or literal
                    (nextindicator, nextdata) = parse_seq[parse_index+1]
                    if nextindicator is None:
                       # search for literal
                       last = s.find(nextdata, currentindex)
                       if last<currentindex:
                          raise ValueError("couldn't terminate wild with lit "+repr(currentindex))
                    else:
                       # data is a re, search for it
                       last = nextdata.search(s, currentindex)
                       if last<currentindex:
                          raise ValueError("couldn't terminate wild with re "+repr(currentindex))
              elif indicator == single_char:
                 # data is length to eat
                 last = currentindex + data
              else:
                 # other directives are always regular expressions
                 last = data.match(s, currentindex) + currentindex
                 if last<currentindex:
                    raise ValueError("couldn't match re at "+repr(currentindex))
              #print("accepting", s[currentindex:last])
              result[current_directive_index] = s[currentindex:last]
              current_directive_index = current_directive_index+1
              currentindex = last
       # sanity check
       if current_directive_index != ndirectives:
          raise SystemError("not enough directives found?")
       return (result, currentindex)

# some useful regular expressions
USERNAMEREGEX = \
  "["+ascii_letters+"]["+ascii_letters+string.digits+"_]*"
STRINGLITREGEX = "'[^\n']*'"
SIMPLEINTREGEX = "["+string.digits+"]+"
id = re.compile(USERNAMEREGEX)
str = re.compile(STRINGLITREGEX)
int = re.compile(SIMPLEINTREGEX)

def test():
    global T, T1, T2, T3

    T = Template("(NNN)NNN-NNNN X X", "X", "N")
    print(T.PARSE("(908)949-2726 Aaron Watters"))

    T1 = Template("s --> s blah", s=str)
    s = "' <-- a string --> ' --> 'blah blah another string blah' blah"
    print(T1.PARSE(s))

    T2 = Template("s --> NNNiX", "X", "N", s=str, i=int)
    print(T2.PARSE("'A STRING' --> 15964653alpha beta gamma"))

    T3 = Template("XsXi", "X", "N", s=str, i=int)
    print(T3.PARSE("prefix'string'interior1234junk not parsed"))

    T4 = Template("MMDDYYX", "X", "MDY")
    print(T4.PARSE("122961 Somebody's birthday!"))


if __name__=="__main__": test()



#!/usr/bin/env python
from reportlab.lib.testutils import setOutDir,makeSuiteForClasses, outputfile, printLocation
setOutDir(__name__)
import sys
from xml.dom import minidom
from xml.sax._exceptions import SAXReaderNotAvailable
import unittest
from reportlab.graphics.shapes import *
from reportlab.graphics import renderSVG

def warnIgnoredRestofTest():
    "Raise a warning (if possible) about a not fully completed test."

    version = sys.version_info[:2]
    msg = "XML parser not found - consider installing expat! Rest of test(s) ignored!"
    if version >= (2, 1):
        import warnings
        warnings.warn(msg)
    else:
        # should better also be printed only once...
        print(msg)

# Check if we have a default XML parser available or not.
try:
    import xml
    from xml.sax import make_parser
    p = xml.sax.make_parser()
    HAVE_XML_PARSER = 1
except SAXReaderNotAvailable:
    HAVE_XML_PARSER = 0

def load(path):
    "Helper function to read the generated SVG again."

    doc = minidom.parse(path)
    doc.normalize()
    return doc.documentElement

class RenderSvgSimpleTestCase(unittest.TestCase):
    "Testing renderSVG module."

    def test0(self):
        "Test two strings in drawing."

        path = outputfile("test_renderSVG_simple_test0.svg")

        d = Drawing(200, 100)
        d.add(String(0, 0, "foo"))
        d.add(String(100, 0, "bar"))
        renderSVG.drawToFile(d, path)

        if not HAVE_XML_PARSER:
            warnIgnoredRestofTest()
            return

        svg = load(path)
        fg = svg.getElementsByTagName('g')[0]           # flipping group
        dg = fg.getElementsByTagName('g')[0]            # diagram group
        textChildren = dg.getElementsByTagName('text')  # text nodes
        t0 = textChildren[0].childNodes[0].nodeValue.strip()
        t1 = textChildren[1].childNodes[0].nodeValue.strip()
        assert t0 == 'foo'
        assert t1 == 'bar'

    def test1(self):
        "Test two strings in group in drawing."

        path = outputfile("test_renderSVG_simple_test1.svg")

        d = Drawing(200, 100)
        g = Group()
        g.add(String(0, 0, "foo"))
        g.add(String(100, 0, "bar"))
        d.add(g)
        renderSVG.drawToFile(d, path)

        if not HAVE_XML_PARSER:
            warnIgnoredRestofTest()
            return

        svg = load(path)
        fg = svg.getElementsByTagName('g')[0]           # flipping group
        dg = fg.getElementsByTagName('g')[0]            # diagram group
        g = dg.getElementsByTagName('g')[0]             # custom group
        textChildren = g.getElementsByTagName('text')   # text nodes
        t0 = textChildren[0].childNodes[0].nodeValue.strip()
        t1 = textChildren[1].childNodes[0].nodeValue.strip()

        assert t0 == 'foo'
        assert t1 == 'bar'

    def test2(self):
        "Test two strings in transformed group in drawing."

        path = outputfile("test_renderSVG_simple_test2.svg")

        d = Drawing(200, 100)
        g = Group()
        g.add(String(0, 0, "foo"))
        g.add(String(100, 0, "bar"))
        g.scale(1.5, 1.2)
        g.translate(50, 0)
        d.add(g)
        renderSVG.drawToFile(d, path)

        if not HAVE_XML_PARSER:
            warnIgnoredRestofTest()
            return

        svg = load(path)
        fg = svg.getElementsByTagName('g')[0]           # flipping group
        dg = fg.getElementsByTagName('g')[0]            # diagram group
        g = dg.getElementsByTagName('g')[0]             # custom group
        textChildren = g.getElementsByTagName('text')   # text nodes
        t0 = textChildren[0].childNodes[0].nodeValue.strip()
        t1 = textChildren[1].childNodes[0].nodeValue.strip()

        assert t0 == 'foo'
        assert t1 == 'bar'

    def test3(self):
        from reportlab.lib.units import cm
        from reportlab.lib import colors

        width=300
        height=60

        #Create fairly simple drawing object,
        drawing=Drawing(width, height)

        p=ArcPath(strokeColor=colors.darkgreen,
                          fillColor=colors.green,
                          hrefURL="http://en.wikipedia.org/wiki/Vector_path",
                          hrefTitle="This big letter C is actually a closed vector path.",
                          strokewidth=0)
        p.addArc(1*cm, 1*cm, 0.8*cm, 20, 340, moveTo=True)
        p.addArc(1*cm, 1*cm, 0.9*cm, 20, 340, reverse=True)
        p.closePath()
        drawing.add(p)

        drawing.add(Rect(2.25*cm, 0.1*cm, 1.5*cm, 0.8*cm, rx=0.25*cm, ry=0.25*cm,

        hrefURL="http://en.wikipedia.org/wiki/Rounded_rectangle",
                               hrefTitle="Rounded Rectangle",
                               strokeColor=colors.red,
                               fillColor=colors.yellow))

        drawing.add(String(1*cm, 1*cm, "Hello World!",
                                 hrefURL="http://en.wikipedia.org/wiki/Hello_world",
                                 hrefTitle="Why 'Hello World'?",
                                 fillColor=colors.darkgreen))
        drawing.add(Rect(4.5*cm, 0.5*cm, 5*cm, 1*cm,
                                hrefURL="http://en.wikipedia.org/wiki/Rectangle",
                                hrefTitle="Wikipedia page on rectangles",
                                strokeColor=colors.blue,
                                fillColor=colors.red))
        drawing.add(Ellipse(7*cm, 1*cm, 2*cm, 0.95*cm,
                                  hrefURL="http://en.wikipedia.org/wiki/Ellipse",
                                  strokeColor=colors.black,
                                  fillColor=colors.yellow))
        drawing.add(Circle(7*cm, 1*cm, 0.9*cm,
                                  hrefURL="http://en.wikipedia.org/wiki/Circle",
                                 strokeColor=colors.black,
                                 fillColor=colors.brown))
        drawing.add(Ellipse(7*cm, 1*cm, 0.5*cm, 0.9*cm,
                                  hrefTitle="Tooltip with no link?",
                                  strokeColor=colors.black,
                                  fillColor=colors.black))
        drawing.add(Polygon([4.5*cm, 1.25*cm, 5*cm, 0.1*cm, 4*cm, 0.1*cm],
                                  hrefURL="http://en.wikipedia.org/wiki/Polygon",
                                  hrefTitle="This triangle is a simple polygon.",
                                  strokeColor=colors.darkgreen,
                                  fillColor=colors.green))

        renderSVG.drawToFile(drawing, outputfile("test_renderSVG_simple_test3.svg"))

    def test4(self):
        "Test character encoding."

        path = outputfile("test_renderSVG_simple_test4.svg")
        specialChar = u'\u2019'

        d = Drawing(200, 100)
        d.add(String(0, 0, "foo"+specialChar))
        d.add(String(100, 0, "bar"))
        renderSVG.drawToFile(d, path)

        if not HAVE_XML_PARSER:
            warnIgnoredRestofTest()
            return

        svg = load(path)
        fg = svg.getElementsByTagName('g')[0]           # flipping group
        dg = fg.getElementsByTagName('g')[0]            # diagram group
        textChildren = dg.getElementsByTagName('text')  # text nodes
        t0 = textChildren[0].childNodes[0].nodeValue.strip()
        t1 = textChildren[1].childNodes[0].nodeValue.strip()
        assert t0 == 'foo'+specialChar, "%s should equal %s" % (ascii(t0),ascii('foo'+specialChar))
        assert t1 == 'bar'

    def test5(self):
        '''tests drawToString inspired by https://bitbucket.org/egillet/ 
        & https://bitbucket.org/johanndt/'''
        d = Drawing(1,1)
        self.assertTrue(isStr(renderSVG.drawToString(d)),msg='renderSVG.draweToString should return bytes')

    def tearDown(self):
        "When finished, make a little index page to view them in situ"
        
        body = """<html>
    <head><title>renderSVG test output</title></head>
    <body>
        <h1>renderSVG test output in a web page</h1>
        <p>We have four SVG diagrams embedded in this page.  Each is within a cyan-coloured div.
        The first 3 have a native size of 400x200, thus consume a height of 200 pixels on
        the page.  The last is 300x60.</p>

        <div style="background-color:cyan">
            <embed src="test_renderSVG_simple_test0.svg" type="image/svg+xml" />
        </div>
        <hr/>
        <div style="background-color:cyan">
            <embed src="test_renderSVG_simple_test1.svg" type="image/svg+xml" />
        </div>
        <hr/>
        <div style="background-color:cyan">
            <embed src="test_renderSVG_simple_test2.svg" type="image/svg+xml" />
        </div>
        <hr/>
        <div style="background-color:cyan">
            <embed src="test_renderSVG_simple_test3.svg" type="image/svg+xml" />
        </div>

        <hr>
        <p>Test of resizing:  the ones below are sized 50%, 100%, 150%. We did this by explicitly setting
        the width and height in the <code>embed</code> tag.</p>
        <div style="background-color:cyan">
            <embed src="test_renderSVG_simple_test3.svg" type="image/svg+xml" width="150" height="45"/>
        </div>
        <hr/>
        <div style="background-color:cyan">
            <embed src="test_renderSVG_simple_test3.svg" type="image/svg+xml" width="300" height="60"/>
        </div>
        <hr/>
        <div style="background-color:cyan">
            <embed src="test_renderSVG_simple_test3.svg" type="image/svg+xml" width="450" height="90"/>
        </div>
        <hr/>

        <p>Test of resizing again:  the ones below are sized 50%, 100%, 150% by setting width only.</p>
        <div style="background-color:cyan">
            <embed src="test_renderSVG_simple_test3.svg" type="image/svg+xml" width="150"/>
        </div>
        <hr/>
        <div style="background-color:cyan">
            <embed src="test_renderSVG_simple_test3.svg" type="image/svg+xml" width="300"/>
        </div>
        <hr/>
        <div style="background-color:cyan">
            <embed src="test_renderSVG_simple_test3.svg" type="image/svg+xml" width="450"/>
        </div>
        <hr/>

        
    </body>
<html>
"""
        with open('test_renderSVG_output.html', 'w') as f:
            f.write(body)

class RenderSvgAxesTestCase(unittest.TestCase):
    "Testing renderSVG module on Axes widgets."

    def test0(self):
        "Test two strings in drawing."

        path = outputfile("axestest0.svg")
        from reportlab.graphics.charts.axes import XCategoryAxis

        d = XCategoryAxis().demo()
        renderSVG.drawToFile(d, path)

def makeSuite():
    return makeSuiteForClasses(RenderSvgSimpleTestCase, RenderSvgAxesTestCase)

#noruntests
if __name__ == "__main__":
    unittest.TextTestRunner().run(makeSuite())
    printLocation()
